"""
Commands:
.autoop add/del/status/list
.op

Nick-format: nick!ident@example.org

Supports adding/deleting multiple autoop-rules:
.autoop add #testchannel firstNick!ident@example.org secNick!ident@example.org

In add/del #channel needed, if changes done in query. If changes are made in #channel, defaults to current #channel.

config-file in root/modules/module_autooop.conf
config-format:
 'nick!ident@example.org':
 - '#channelOne'
 - '#channelTwo'

 'secNick!ident@example.org':
 - '#channelTwo'
"""

import fnmatch
import yaml
import os.path
import sys
import re

oplistfile = os.path.join(sys.path[0], "modules", "module_autoop.conf")
oplist = dict()


def init(botconfig):
    global oplist
    if os.path.exists(oplistfile):
        oplist = yaml.load(file(oplistfile), Loader=yaml.FullLoader)


def writeConfig():
    # open file for writing (note, removes all markings that have been made to config-file prior to .rehash
    f = open(oplistfile, "w")
    yaml.dump(oplist, f, default_flow_style=False)
    f.close()


def isAutoOppable(user, channel):
    for pattern in oplist:
        if fnmatch.fnmatch(user, pattern):
            if channel in oplist[pattern]:
                return True


def handle_userJoined(bot, user, channel):
    if isAdmin(user) or isAutoOppable(user, channel):
        nick = getNick(user)
        bot.log("auto-opping %s on %s" % (user, channel))
        bot.mode(channel, True, "o", user=nick)


def handle_args(args, channel):
    message = ""
    nicks = list()
    rg = re.compile(
        ".*?(!).*?(@).*?", re.IGNORECASE | re.DOTALL
    )  # regex for validating nick format
    if re.match(rg, args[0].strip()) is None and args[0].strip()[0] == "#":
        op_channel = args[0].strip()
        for nick in args[1:]:
            if re.match(rg, nick):
                nicks.append(nick)
            else:
                message += "%s not in correct format. " % nick
    else:
        op_channel = channel
        for nick in args[0:]:
            if re.match(rg, nick):
                nicks.append(nick)
            else:
                message += "%s not in correct format. " % nick
    return nicks, op_channel, message


def autoop_status(user, channel, args):
    nick = getNick(user)
    if isAdmin(user):
        return "%s: you are %s on %s, status: bot admin" % (nick, user, channel)
    elif isAutoOppable(user, channel):
        return "%s: you are %s on %s, status: auto-op" % (nick, user, channel)
    else:
        return "%s: you are %s on %s: status: -" % (nick, user, channel)


def autoop_add(user, channel, args):
    if isAdmin(user):
        nicks = list()
        message = ""
        if len(args) > 0:
            nicks, op_channel, message = handle_args(args, channel)
            if cmp(op_channel, user) != 0:
                for nick in nicks:
                    if (
                        nick in oplist
                    ):  # if already in oplist, just add to existing list
                        if (
                            op_channel in oplist[nick]
                        ):  # if not in nicks channel list, add
                            message += "Auto-op for %s already in %s. " % (
                                nick,
                                op_channel,
                            )
                        else:
                            message += "Auto-op for %s in %s added. " % (
                                nick,
                                op_channel,
                            )
                            oplist[nick].append(op_channel)
                    else:
                        message += "Auto-op for %s in %s added. " % (nick, op_channel)
                        oplist[nick] = list()
                        oplist[nick].append(op_channel)
                writeConfig()
            else:
                message = "#channel always needed when making changes in query!"
            return message
        else:
            return "Usage: .autoop add [#channel] nick!ident@hostma.sk"


def autoop_del(user, channel, args):
    if isAdmin(user):
        message = ""
        if len(args) > 0:
            nicks, op_channel, message = handle_args(args, channel)
            if cmp(op_channel, user) != 0:
                for nick in nicks:
                    if nick in oplist:
                        if (
                            op_channel in oplist[nick]
                        ):  # if in nicks channel list, remove
                            message += "Auto-op for %s in %s removed. " % (
                                nick,
                                op_channel,
                            )
                            oplist[nick].remove(op_channel)
                            if len(oplist[nick]) == 0:
                                del oplist[nick]
                        else:
                            message += "No auto-op for %s in %s. " % (nick, op_channel)
                    else:
                        message += "No auto-op for %s in %s. " % (nick, op_channel)
                writeConfig()
            else:
                message = "#channel always needed when making changes in query!"
            return message
        else:
            return "Usage: .autoop del [#channel] nick!ident@hostma.sk"


def autoop_list(user, channel, args):
    if isAdmin(user):
        i = 0
        rg = re.compile(
            ".*?(!).*?(@).*?", re.IGNORECASE | re.DOTALL
        )  # regex for validating nick format
        if (
            len(args) == 0
            or (re.match(rg, args[0].strip()) is None and args[0].strip()[0] == "#")
        ) and args[0] != "me":
            if len(args) == 0:
                op_channel = channel
            elif re.match(rg, args[0].strip()) is None and args[0].strip()[0] == "#":
                op_channel = args[0].strip()
            message = "Auto-ops in %s: " % op_channel
            for nick, channels in oplist.items():
                for chan in channels:
                    if chan == op_channel:
                        message += nick + " "
                        i = i + 1
            if i != 0:
                return message
            else:
                return "%s has no auto-ops." % op_channel
        elif re.match(rg, args[0].strip()) or args[0] == "me":
            if args[0] != "me":
                nick = args[0].strip()
            else:
                nick = user
            message = "%s auto-op in: " % nick
            if nick in oplist:
                for chan in oplist[nick]:
                    message += chan + " "
                    i = i + 1
            if i != 0:
                return message
            else:
                return "%s not auto-op anywhere." % nick
        else:
            return "Usage: .autoop_list [#channel _or_ nick!ident@hostma.sk]"
    else:
        nick = user
        i = 0
        message = "%s auto-op in: " % nick
        if nick in oplist:
            for chan in oplist[nick]:
                message += chan + " "
                i = i + 1
        if i != 0:
            return message
        else:
            return "%s not auto-op anywhere." % nick


def make_valid_commands_msg(commands):
    message = "Valid commands are: "
    for command in commands:
        message += command
        message += ", "
    message = message[:-2]
    message += "."
    return message


def command_op(bot, user, channel, args):
    if isAdmin(user) or isAutoOppable(user, channel):
        nick = getNick(user)
        bot.log("opping %s by request on %s" % (user, channel))
        bot.mode(channel, True, "o", user=nick)


def command_autoop(bot, user, channel, args):
    commands = {
        "add": autoop_add,
        "del": autoop_del,
        "list": autoop_list,
        "status": autoop_status,
    }
    args = args.split()
    if len(args) > 0:
        command = args[0]
        if command in commands:
            args.remove(command)
            message = commands[command](user, channel, args)
        else:
            message = make_valid_commands_msg(commands)
    else:
        message = make_valid_commands_msg(commands)
    return bot.say(channel, message)



# -*- coding: utf-8 -*-
from __future__ import print_function, division, unicode_literals
import logging
from datetime import date, datetime, timedelta


log = logging.getLogger("openweather")
default_location = "Helsinki"
threshold = 120

appid = None


def init(bot):
    global appid
    global default_location
    global threshold

    config = bot.config.get("module_openweather", {})
    default_location = config.get("default_location", "Helsinki")
    log.info("Using %s as default location" % default_location)

    appid = config.get("appid")
    if appid:
        log.info("Using OpenWeatherMap appid %s" % appid)
    else:
        log.warning("Appid not found!")

    threshold = int(
        config.get("threshold", 120)
    )  # threshold to show measuring time in minutes


def command_weather(bot, user, channel, args):
    """Weather from openweathermap"""
    if not appid:
        log.warn("No OpenWeatherMap appid set in configuration")
        return False

    if args:
        location = args
    else:
        location = default_location

    url = "http://api.openweathermap.org/data/2.5/weather?q=%s&units=metric&appid=%s"
    r = bot.get_url(url % (location, appid))

    try:
        data = r.json()
    except:
        log.debug("Couldn't parse JSON.")
        return bot.say(channel, "Error: API error, unable to parse JSON response.")

    if "cod" not in data or int(data["cod"]) != 200:
        log.debug("status != 200")
        return bot.say(channel, "Error: API error.")

    if "name" not in data:
        return bot.say(channel, "Error: Location not found.")
    if "main" not in data:
        return bot.say(channel, "Error: Unknown error.")

    location = "%s, %s" % (data["name"], data["sys"]["country"])
    main = data["main"]

    old_data = False
    if "dt" in data:
        measured = datetime.utcfromtimestamp(data["dt"])
        if datetime.utcnow() - timedelta(minutes=threshold) > measured:
            old_data = True
    if old_data:
        text = "%s (%s UTC): " % (location, measured.strftime("%Y-%m-%d %H:%M"))
    else:
        text = "%s: " % location

    if "temp" not in main:
        return bot.say(channel, "Error: Data not found.")

    temperature = main["temp"]  # temperature converted from kelvin to celcius
    text += "Temperature: %.1f°C" % temperature

    if "wind" in data and "speed" in data["wind"]:
        wind = data["wind"]["speed"]  # Wind speed in mps (m/s)

        feels_like = (
            13.12
            + 0.6215 * temperature
            - 13.956 * (wind**0.16)
            + 0.4867 * temperature * (wind**0.16)
        )
        text += ", feels like: %.1f°C" % feels_like
        text += ", wind: %.1f m/s" % wind

    if "humidity" in main:
        humidity = main["humidity"]  # Humidity in %
        text += ", humidity: %d%%" % humidity
    if "pressure" in main:
        pressure = main["pressure"]  # Atmospheric pressure in hPa
        text += ", pressure: %d hPa" % pressure
    if "clouds" in data and "all" in data["clouds"]:
        cloudiness = data["clouds"]["all"]  # Cloudiness in %
        text += ", cloudiness: %d%%" % cloudiness

    return bot.say(channel, text)


def command_forecast(bot, user, channel, args):
    if not appid:
        log.warn("No OpenWeatherMap appid set in configuration")
        return False

    if args:
        location = args
    else:
        location = default_location

    url = "http://api.openweathermap.org/data/2.5/forecast/daily?q=%s&cnt=5&mode=json&units=metric&appid=%s"
    r = bot.get_url(url % (location, appid))

    try:
        data = r.json()
    except:
        log.debug("Couldn't parse JSON.")
        return bot.say(channel, "Error: API error, unable to parse JSON response.")

    if "cod" not in data or int(data["cod"]) != 200:
        log.debug("status != 200")
        return bot.say(channel, "Error: API error.")

    if "city" not in data or "name" not in data["city"]:
        return bot.say(channel, "Error: Location not found.")

    if not data["list"]:
        return bot.say(channel, "Error: No forecast data.")

    text = "%s, %s: " % (data["city"]["name"], data["city"]["country"])

    cur_date = date.today()
    forecast_text = []
    for d in data["list"]:
        forecast_date = date.fromtimestamp(d["dt"])
        date_delta = (forecast_date - cur_date).days

        if date_delta < 1:
            continue

        if date_delta == 1:
            forecast_text.append(
                "tomorrow: %.1f - %.1f °C (%s)"
                % (d["temp"]["min"], d["temp"]["max"], d["weather"][0]["description"])
            )
        else:
            forecast_text.append(
                "in %d days: %.1f - %.1f °C (%s)"
                % (
                    date_delta,
                    d["temp"]["min"],
                    d["temp"]["max"],
                    d["weather"][0]["description"],
                )
            )

        if len(forecast_text) >= 3:
            break

    text += ", ".join(forecast_text)
    return bot.say(channel, text)



# -*- coding: utf-8 -*-
import logging
from nose.tools import eq_
import bot_mock
from pyfibot.modules import module_urltitle
from utils import check_re

import pytest
from vcr import VCR

my_vcr = VCR(
    path_transformer=VCR.ensure_suffix(".yaml"),
    cassette_library_dir="tests/cassettes/",
    record_mode=pytest.config.getoption("--vcrmode"),
)

logging.basicConfig()  # you need to initialize logging, otherwise you will not see anything from vcrpy
vcr_log = logging.getLogger("vcr")
vcr_log.setLevel(logging.DEBUG)


@pytest.fixture
def botmock():
    bot = bot_mock.BotMock()
    module_urltitle.init(bot)
    return bot


lengh_str_regex = r"\d+(h|m|s)(\d+(m))?(\d+s)?"
views_str_regex = r"\d+(\.\d+)?(k|M|Billion|Trillion)?"
age_str_regex = r"(FRESH|(\d+(\.\d+)?(y|d) ago))"


def __handle_url(botmock, msg):
    res = module_urltitle.handle_url(botmock, None, "#channel", msg, msg)
    if type(res) == tuple:
        return res[1]
    else:
        return res


@my_vcr.use_cassette
def test_spotify_ignore(botmock):
    msg = "http://open.spotify.com/artist/4tuiQRw9bC9HZhSFJEJ9Mz"
    assert __handle_url(botmock, msg) is None


@my_vcr.use_cassette
def test_manual_ignore(botmock):
    msg = "- foofoo http://www.youtube.com/"
    assert __handle_url(botmock, msg) is None


@my_vcr.use_cassette
def test_iltalehti(botmock):
    msg = "https://www.iltalehti.fi/ulkomaat/a/2013072917307393"
    assert "Title: Saksassa syntyi jättivauva - yli kuusi kiloa!" == __handle_url(
        botmock, msg
    )


@my_vcr.use_cassette
def test_iltasanomat(botmock):
    msg = "https://www.is.fi/tv-ja-elokuvat/art-2000007925056.html"
    assert (
        "Title: The Voice of Finlandin finalistinelikko on selvill\xe4!"
        == __handle_url(botmock, msg)
    )


@my_vcr.use_cassette
def test_stackoverflow(botmock):
    msg = "http://stackoverflow.com/questions/6905508/python-search-html-document-for-capital-letters"
    assert (
        "Title: Python search HTML document for capital letters - 0pts - python/regex/coda/letters/capitalize"
        == __handle_url(botmock, msg)
    )


@my_vcr.use_cassette
def test_vimeo(botmock):
    regex = "Title: (.*?) by (.*?) \[%s - %s likes - %s views - %s]" % (
        lengh_str_regex,
        views_str_regex,
        views_str_regex,
        age_str_regex,
    )
    msg = "http://vimeo.com/29996808"
    check_re(regex, __handle_url(botmock, msg))


@my_vcr.use_cassette
def test_ebay_cgi(botmock):
    msg = "http://cgi.ebay.co.uk/ws/eBayISAPI.dll?ViewItem&item=121136837564"
    regex = "Title: .*? \[\d+\.\de - over \d+ available - ships from .*?\ - ENDED]"
    check_re(regex, __handle_url(botmock, msg))


@my_vcr.use_cassette
def test_steamstore(botmock):
    msg = "http://store.steampowered.com/app/440/"
    title = "Title: Team Fortress 2 on Steam"
    eq_(title, __handle_url(botmock, msg))


@my_vcr.use_cassette
def test_pythonorg(botmock):
    msg = "http://python.org"
    eq_("Title: Welcome to Python.org", __handle_url(botmock, msg))


@my_vcr.use_cassette
def test_github(botmock):
    msg = "https://github.com/lepinkainen/pyfibot"
    assert "Title: lepinkainen/pyfibot" == __handle_url(botmock, msg)


# Test ignored titles
@my_vcr.use_cassette
def test_apina(botmock):
    msg = "http://apina.biz"
    assert __handle_url(botmock, msg) is None


@my_vcr.use_cassette
def test_poliisifi(botmock):
    msg = "http://poliisi.fi/tietoa_poliisista/tiedotteet/1/1/merisotakoulun_upseerikokelas_menehtyi_ampuharjoituksessa_hangon_syndalenissa_42596"
    assert __handle_url(botmock, msg) is None



# -*- coding: utf-8 -*-
import pytest
import sys
import os
import json
import tempfile
from unittest.mock import patch, mock_open

# Add the project root to Python path
sys.path.insert(0, os.path.join(os.path.dirname(__file__), ".."))

from pyfibot.pyfibot import validate_config, read_config


class TestConfigValidation:
    """Test configuration validation functionality"""

    def test_minimal_valid_config(self):
        """Test validation with minimal valid configuration"""
        config = {
            "networks": [
                {
                    "alias": "testnet",
                    "address": ["irc.example.com", 6667],
                    "nick": "testbot"
                }
            ]
        }
        
        # Mock the schema file reading
        mock_schema = {
            "type": "object",
            "required": ["networks"],
            "properties": {
                "networks": {
                    "type": "array",
                    "items": {
                        "type": "object",
                        "required": ["alias", "address", "nick"],
                        "properties": {
                            "alias": {"type": "string"},
                            "address": {"type": "array"},
                            "nick": {"type": "string"}
                        }
                    }
                }
            }
        }
        
        with patch("builtins.open", mock_open(read_data=json.dumps(mock_schema))):
            with patch("sys.path", ["/fake/path"]):
                assert validate_config(config) is True

    def test_missing_required_field(self):
        """Test validation with missing required field"""
        config = {
            # Missing required "networks" field
            "nick": "testbot"
        }
        
        mock_schema = {
            "type": "object",
            "required": ["networks"],
            "properties": {
                "networks": {"type": "array"}
            }
        }
        
        with patch("builtins.open", mock_open(read_data=json.dumps(mock_schema))):
            with patch("sys.path", ["/fake/path"]):
                assert validate_config(config) is False

    def test_invalid_type(self):
        """Test validation with invalid field type"""
        config = {
            "networks": "not_an_array"  # Should be array
        }
        
        mock_schema = {
            "type": "object",
            "required": ["networks"],
            "properties": {
                "networks": {"type": "array"}
            }
        }
        
        with patch("builtins.open", mock_open(read_data=json.dumps(mock_schema))):
            with patch("sys.path", ["/fake/path"]):
                assert validate_config(config) is False

    def test_valid_config_with_optional_fields(self):
        """Test validation with valid config including optional fields"""
        config = {
            "networks": [
                {
                    "alias": "testnet",
                    "address": ["irc.example.com", 6667],
                    "nick": "testbot"
                }
            ],
            "admins": ["admin!user@host.com"],
            "nick": "globalbot",
            "logging": {
                "debug": True
            }
        }
        
        mock_schema = {
            "type": "object",
            "required": ["networks"],
            "properties": {
                "networks": {
                    "type": "array",
                    "items": {
                        "type": "object",
                        "required": ["alias", "address", "nick"],
                        "properties": {
                            "alias": {"type": "string"},
                            "address": {"type": "array"},
                            "nick": {"type": "string"}
                        }
                    }
                },
                "admins": {
                    "type": "array",
                    "items": {"type": "string"}
                },
                "nick": {"type": "string"},
                "logging": {
                    "type": "object",
                    "properties": {
                        "debug": {"type": "boolean"}
                    }
                }
            }
        }
        
        with patch("builtins.open", mock_open(read_data=json.dumps(mock_schema))):
            with patch("sys.path", ["/fake/path"]):
                assert validate_config(config) is True

    def test_admin_pattern_validation(self):
        """Test admin pattern validation from real schema"""
        config = {
            "networks": [{"alias": "test", "address": ["test.com", 6667], "nick": "bot"}],
            "admins": ["validuser!validident@validhost.com"]
        }
        
        # Use actual pattern from schema
        mock_schema = {
            "type": "object",
            "required": ["networks"],
            "properties": {
                "networks": {"type": "array"},
                "admins": {
                    "type": "array",
                    "items": {
                        "type": "string",
                        "pattern": "^.+!.+@.+$"
                    }
                }
            }
        }
        
        with patch("builtins.open", mock_open(read_data=json.dumps(mock_schema))):
            with patch("sys.path", ["/fake/path"]):
                assert validate_config(config) is True

    def test_invalid_admin_pattern(self):
        """Test invalid admin pattern"""
        config = {
            "networks": [{"alias": "test", "address": ["test.com", 6667], "nick": "bot"}],
            "admins": ["invalid_format"]  # Missing ! and @ separators
        }
        
        mock_schema = {
            "type": "object",
            "required": ["networks"],
            "properties": {
                "networks": {"type": "array"},
                "admins": {
                    "type": "array",
                    "items": {
                        "type": "string",
                        "pattern": "^.+!.+@.+$"
                    }
                }
            }
        }
        
        with patch("builtins.open", mock_open(read_data=json.dumps(mock_schema))):
            with patch("sys.path", ["/fake/path"]):
                assert validate_config(config) is False

    def test_schema_file_not_found(self):
        """Test behavior when schema file is not found"""
        config = {"networks": []}
        
        with patch("builtins.open", side_effect=FileNotFoundError):
            with pytest.raises(FileNotFoundError):
                validate_config(config)

    def test_invalid_json_schema(self):
        """Test behavior with invalid JSON schema file"""
        config = {"networks": []}
        
        with patch("builtins.open", mock_open(read_data="invalid json")):
            with patch("sys.path", ["/fake/path"]):
                with pytest.raises(json.JSONDecodeError):
                    validate_config(config)

    def test_complex_network_config(self):
        """Test validation of complex network configuration"""
        config = {
            "networks": [
                {
                    "alias": "freenode",
                    "address": ["irc.libera.chat", 6697],
                    "nick": "pyfibot",
                    "channels": ["#test", "#dev"],
                    "ssl": True,
                    "password": "secret"
                },
                {
                    "alias": "local",
                    "address": ["localhost", 6667],
                    "nick": "localbot",
                    "channels": ["#general"]
                }
            ]
        }
        
        mock_schema = {
            "type": "object",
            "required": ["networks"],
            "properties": {
                "networks": {
                    "type": "array",
                    "items": {
                        "type": "object",
                        "required": ["alias", "address", "nick"],
                        "properties": {
                            "alias": {"type": "string"},
                            "address": {"type": "array"},
                            "nick": {"type": "string"},
                            "channels": {
                                "type": "array",
                                "items": {"type": "string"}
                            },
                            "ssl": {"type": "boolean"},
                            "password": {"type": "string"}
                        }
                    }
                }
            }
        }
        
        with patch("builtins.open", mock_open(read_data=json.dumps(mock_schema))):
            with patch("sys.path", ["/fake/path"]):
                assert validate_config(config) is True


class TestConfigReading:
    """Test configuration file reading functionality"""

    def test_read_config_yaml_file(self):
        """Test reading a YAML configuration file"""
        yaml_content = """
networks:
  - alias: testnet
    address: [irc.example.com, 6667]
    nick: testbot
    channels:
      - "#test"
admins:
  - "admin!user@host.com"
nick: globalbot
"""
        
        with tempfile.NamedTemporaryFile(mode='w', suffix='.yml', delete=False) as f:
            f.write(yaml_content)
            f.flush()
            
            try:
                with patch("sys.argv", ["pyfibot.py", f.name]):
                    config = read_config()
                    
                    assert "networks" in config
                    assert len(config["networks"]) == 1
                    assert config["networks"][0]["alias"] == "testnet"
                    assert config["admins"] == ["admin!user@host.com"]
                    assert config["nick"] == "globalbot"
            finally:
                os.unlink(f.name)

    def test_read_config_missing_file(self):
        """Test reading non-existent configuration file"""
        with patch("sys.argv", ["pyfibot.py", "nonexistent.yml"]):
            with pytest.raises(SystemExit):
                read_config()

    def test_read_config_invalid_yaml(self):
        """Test reading invalid YAML file"""
        invalid_yaml = """
networks:
  - alias: testnet
    address: [irc.example.com, 6667
    # Missing closing bracket - invalid YAML
"""
        
        with tempfile.NamedTemporaryFile(mode='w', suffix='.yml', delete=False) as f:
            f.write(invalid_yaml)
            f.flush()
            
            try:
                with patch("sys.argv", ["pyfibot.py", f.name]):
                    with pytest.raises(SystemExit):
                        read_config()
            finally:
                os.unlink(f.name)

    def test_read_config_no_args(self):
        """Test reading config when no config file is specified"""
        with patch("sys.argv", ["pyfibot.py"]):
            with pytest.raises(SystemExit):
                read_config()

    def test_read_config_empty_file(self):
        """Test reading empty configuration file"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='.yml', delete=False) as f:
            f.write("")  # Empty file
            f.flush()
            
            try:
                with patch("sys.argv", ["pyfibot.py", f.name]):
                    config = read_config()
                    # Empty YAML file should return None, which becomes empty dict
                    assert config is None or config == {}
            finally:
                os.unlink(f.name)

    def test_read_config_yaml_with_comments(self):
        """Test reading YAML file with comments"""
        yaml_content = """
# PyFiBot configuration file
networks:
  # Main network
  - alias: testnet
    address: [irc.example.com, 6667]  # Standard IRC port
    nick: testbot
    channels:
      - "#test"  # Test channel

# Admin users (nick!user@host format)
admins:
  - "admin!user@host.com"

# Global bot nickname
nick: globalbot
"""
        
        with tempfile.NamedTemporaryFile(mode='w', suffix='.yml', delete=False) as f:
            f.write(yaml_content)
            f.flush()
            
            try:
                with patch("sys.argv", ["pyfibot.py", f.name]):
                    config = read_config()
                    
                    assert "networks" in config
                    assert config["networks"][0]["alias"] == "testnet"
                    assert config["admins"] == ["admin!user@host.com"]
            finally:
                os.unlink(f.name)


class TestConfigEdgeCases:
    """Test edge cases in configuration handling"""

    def test_unicode_in_config(self):
        """Test configuration with unicode characters"""
        yaml_content = """
networks:
  - alias: tëstnet
    address: [irc.exämple.com, 6667]
    nick: tëstbot
    channels:
      - "#tëst"
admins:
  - "ädmin!üser@höst.com"
"""
        
        with tempfile.NamedTemporaryFile(mode='w', suffix='.yml', delete=False, encoding='utf-8') as f:
            f.write(yaml_content)
            f.flush()
            
            try:
                with patch("sys.argv", ["pyfibot.py", f.name]):
                    config = read_config()
                    
                    assert config["networks"][0]["alias"] == "tëstnet"
                    assert config["networks"][0]["nick"] == "tëstbot"
                    assert config["admins"][0] == "ädmin!üser@höst.com"
            finally:
                os.unlink(f.name)

    def test_large_config_file(self):
        """Test with a large configuration file"""
        # Create config with many networks
        networks = []
        for i in range(50):
            networks.append({
                "alias": f"network_{i}",
                "address": [f"irc{i}.example.com", 6667],
                "nick": f"bot_{i}",
                "channels": [f"#channel_{j}" for j in range(10)]
            })
        
        config_dict = {
            "networks": networks,
            "admins": [f"admin{i}!user{i}@host{i}.com" for i in range(20)]
        }
        
        yaml_content = f"networks:\n"
        for net in networks:
            yaml_content += f"  - alias: {net['alias']}\n"
            yaml_content += f"    address: [{net['address'][0]}, {net['address'][1]}]\n"
            yaml_content += f"    nick: {net['nick']}\n"
            yaml_content += "    channels:\n"
            for chan in net['channels']:
                yaml_content += f"      - \"{chan}\"\n"
        
        yaml_content += "admins:\n"
        for admin in config_dict['admins']:
            yaml_content += f"  - \"{admin}\"\n"
        
        with tempfile.NamedTemporaryFile(mode='w', suffix='.yml', delete=False) as f:
            f.write(yaml_content)
            f.flush()
            
            try:
                with patch("sys.argv", ["pyfibot.py", f.name]):
                    config = read_config()
                    
                    assert len(config["networks"]) == 50
                    assert len(config["admins"]) == 20
                    assert config["networks"][0]["alias"] == "network_0"
                    assert config["networks"][49]["alias"] == "network_49"
            finally:
                os.unlink(f.name)


""" """

from __future__ import unicode_literals, print_function, division
from twisted.internet import reactor

# rejoin after 1 minute
delay = 61


def handle_kickedFrom(bot, channel, kicker, message):
    """Rejoin channel after 60 seconds"""
    bot.log("Kicked by %s from %s. Reason: %s" % (kicker, channel, message))
    bot.log("Rejoining in %d seconds" % delay)
    bot.network.channels.remove(channel)
    # pylint: disable=no-member
    reactor.callLater(delay, bot.join, channel)



# -*- coding: utf-8 -*-
import pytest
import sys
import os

# Add the project root to Python path
sys.path.insert(0, os.path.join(os.path.dirname(__file__), ".."))

from pyfibot.modules.module_bmi import calc_bmi, print_bmi, command_bmi


class TestBMICalculation:
    """Test BMI calculation logic"""

    def test_calc_bmi_normal_values(self):
        """Test BMI calculation with normal values"""
        # 170cm, 70kg = BMI 24.22
        result = calc_bmi(170, 70)
        assert abs(result - 24.22) < 0.01

    def test_calc_bmi_formula_accuracy(self):
        """Test BMI formula accuracy with known values"""
        # BMI = (weight in kg / (height in m)^2)
        # 180cm, 80kg = 80 / (1.8^2) = 80 / 3.24 = 24.69
        result = calc_bmi(180, 80)
        expected = (80 / (1.8 ** 2))
        assert abs(result - expected) < 0.01

    def test_calc_bmi_edge_cases(self):
        """Test BMI calculation with edge cases"""
        # Very tall person
        result = calc_bmi(220, 100)
        expected = (100 / (2.2 ** 2))
        assert abs(result - expected) < 0.01
        
        # Very short person  
        result = calc_bmi(120, 30)
        expected = (30 / (1.2 ** 2))
        assert abs(result - expected) < 0.01

    def test_calc_bmi_float_inputs(self):
        """Test BMI calculation with float inputs"""
        result = calc_bmi(175.5, 72.3)
        expected = (72.3 / (1.755 ** 2))
        assert abs(result - expected) < 0.01

    def test_calc_bmi_string_inputs(self):
        """Test BMI calculation with string inputs (should convert)"""
        result = calc_bmi(float("170"), float("70"))
        expected = (70 / (1.7 ** 2))
        assert abs(result - expected) < 0.01

    def test_calc_bmi_zero_height(self):
        """Test BMI calculation with zero height (should raise error)"""
        with pytest.raises(ZeroDivisionError):
            calc_bmi(0, 70)

    def test_calc_bmi_negative_values(self):
        """Test BMI calculation with negative values"""
        # The function doesn't validate inputs, but let's test behavior
        result = calc_bmi(-170, 70)
        # Actually with -170cm height squared becomes positive, so result is positive
        assert result > 0  # Height squared becomes positive

    def test_calc_bmi_very_large_values(self):
        """Test BMI calculation with very large values"""
        result = calc_bmi(300, 200)
        expected = (200 / (3.0 ** 2))
        assert abs(result - expected) < 0.01


class TestBMICategories:
    """Test BMI categorization logic"""

    def test_severely_underweight(self):
        """Test severely underweight category"""
        result = print_bmi(15.0)
        assert "severely underweight" in result
        assert "15.00" in result

    def test_severely_underweight_boundary(self):
        """Test severely underweight boundary"""
        result = print_bmi(16.4)
        assert "severely underweight" in result
        
        result = print_bmi(16.5)
        assert "underweight (from 16.5 to 18.4)" in result

    def test_underweight(self):
        """Test underweight category"""
        result = print_bmi(17.0)
        assert "underweight (from 16.5 to 18.4)" in result
        assert "17.00" in result

    def test_underweight_boundary(self):
        """Test underweight boundary"""
        result = print_bmi(18.4)
        assert "underweight (from 16.5 to 18.4)" in result
        
        result = print_bmi(18.5)
        assert "normal (from 18.5 to 24.9)" in result

    def test_normal_weight(self):
        """Test normal weight category"""
        result = print_bmi(22.0)
        assert "normal (from 18.5 to 24.9)" in result
        assert "22.00" in result

    def test_normal_weight_boundary(self):
        """Test normal weight boundary"""
        result = print_bmi(24.9)
        assert "normal (from 18.5 to 24.9)" in result
        
        result = print_bmi(25.0)
        assert "overweight (from 25 to 30)" in result

    def test_overweight(self):
        """Test overweight category"""
        result = print_bmi(27.5)
        assert "overweight (from 25 to 30)" in result
        assert "27.50" in result

    def test_overweight_boundary(self):
        """Test overweight boundary"""
        result = print_bmi(30.0)
        assert "overweight (from 25 to 30)" in result
        
        result = print_bmi(30.1)
        assert "obese class I (from 30.1 to 34.9)" in result

    def test_obese_class_i(self):
        """Test obese class I category"""
        result = print_bmi(32.0)
        assert "obese class I (from 30.1 to 34.9)" in result
        assert "32.00" in result

    def test_obese_class_i_boundary(self):
        """Test obese class I boundary"""
        result = print_bmi(34.9)
        assert "obese class I (from 30.1 to 34.9)" in result
        
        result = print_bmi(35.0)
        assert "obese class II (from 35 to 40)" in result

    def test_obese_class_ii(self):
        """Test obese class II category"""
        result = print_bmi(37.5)
        assert "obese class II (from 35 to 40)" in result
        assert "37.50" in result

    def test_obese_class_ii_boundary(self):
        """Test obese class II boundary"""
        result = print_bmi(40.0)
        assert "obese class II (from 35 to 40)" in result
        
        result = print_bmi(40.1)
        assert "obese class III (over 40)" in result

    def test_obese_class_iii(self):
        """Test obese class III category"""
        result = print_bmi(45.0)
        assert "obese class III (over 40)" in result
        assert "45.00" in result

    def test_extreme_values(self):
        """Test with extreme BMI values"""
        result = print_bmi(5.0)
        assert "severely underweight" in result
        
        result = print_bmi(100.0)
        assert "obese class III (over 40)" in result

    def test_decimal_precision(self):
        """Test decimal precision in output"""
        result = print_bmi(23.456789)
        assert "23.46" in result  # Should be rounded to 2 decimal places

    def test_return_format(self):
        """Test that print_bmi returns expected format"""
        result = print_bmi(22.5)
        assert result.startswith("your bmi is ")
        assert " which is " in result
        assert result.endswith("normal (from 18.5 to 24.9)")


class MockBot:
    """Mock bot for testing command function"""
    def __init__(self):
        self.messages = []
    
    def say(self, channel, message):
        self.messages.append((channel, message))
        return (channel, message)


class TestBMICommand:
    """Test the BMI command function"""

    def setup_method(self):
        """Setup test fixtures"""
        self.bot = MockBot()

    def test_command_bmi_valid_input(self):
        """Test BMI command with valid input"""
        result = command_bmi(self.bot, "testuser", "#testchan", "170/70")
        
        assert result[0] == "#testchan"
        assert "24.22" in result[1]
        assert "normal" in result[1]

    def test_command_bmi_invalid_format(self):
        """Test BMI command with invalid format"""
        result = command_bmi(self.bot, "testuser", "#testchan", "170")
        
        assert result[0] == "#testchan"
        assert "Usage: bmi height(cm)/weight(kg)" in result[1]

    def test_command_bmi_too_many_parts(self):
        """Test BMI command with too many parts"""
        result = command_bmi(self.bot, "testuser", "#testchan", "170/70/extra")
        
        assert result[0] == "#testchan"
        assert "Usage: bmi height(cm)/weight(kg)" in result[1]

    def test_command_bmi_empty_args(self):
        """Test BMI command with empty arguments"""
        result = command_bmi(self.bot, "testuser", "#testchan", "")
        
        assert result[0] == "#testchan"
        assert "Usage: bmi height(cm)/weight(kg)" in result[1]

    def test_command_bmi_no_separator(self):
        """Test BMI command without separator"""
        result = command_bmi(self.bot, "testuser", "#testchan", "170 70")
        
        assert result[0] == "#testchan"
        assert "Usage: bmi height(cm)/weight(kg)" in result[1]

    def test_command_bmi_with_spaces(self):
        """Test BMI command with spaces around values"""
        result = command_bmi(self.bot, "testuser", "#testchan", " 170 / 70 ")
        
        # This should work as split() will handle extra spaces
        assert result[0] == "#testchan"
        assert "24.22" in result[1]

    def test_command_bmi_non_numeric_values(self):
        """Test BMI command with non-numeric values"""
        with pytest.raises(ValueError):
            command_bmi(self.bot, "testuser", "#testchan", "abc/def")

    def test_command_bmi_zero_height(self):
        """Test BMI command with zero height"""
        with pytest.raises(ZeroDivisionError):
            command_bmi(self.bot, "testuser", "#testchan", "0/70")

    def test_command_bmi_negative_values(self):
        """Test BMI command with negative values"""
        result = command_bmi(self.bot, "testuser", "#testchan", "-170/70")
        
        # Should still calculate, but result will be normal since height squared is positive
        assert result[0] == "#testchan"
        # The BMI calculation with -170 height still gives normal result
        assert "normal" in result[1]

    def test_command_bmi_float_strings(self):
        """Test BMI command with float string values"""
        # The actual command uses int() which will fail with floats
        with pytest.raises(ValueError):
            command_bmi(self.bot, "testuser", "#testchan", "175.5/72.3")


class TestBMIRealWorldCases:
    """Test BMI with real-world test cases"""

    def test_realistic_adult_males(self):
        """Test BMI for realistic adult male values"""
        test_cases = [
            (180, 75, "normal"),  # Average male
            (175, 65, "normal"),  # Lean male  
            (185, 95, "overweight"),  # Heavy male
            (170, 50, "underweight"),  # Light male - adjusted to actually be underweight
        ]
        
        for height, weight, expected_category in test_cases:
            bmi = calc_bmi(height, weight)
            result = print_bmi(bmi)
            assert expected_category in result

    def test_realistic_adult_females(self):
        """Test BMI for realistic adult female values"""
        test_cases = [
            (165, 60, "normal"),  # Average female
            (160, 50, "normal"),  # Lean female
            (170, 75, "overweight"),  # Heavy female
            (155, 42, "underweight"),  # Light female - adjusted to actually be underweight
        ]
        
        for height, weight, expected_category in test_cases:
            bmi = calc_bmi(height, weight)
            result = print_bmi(bmi)
            assert expected_category in result

    def test_athlete_edge_cases(self):
        """Test BMI for athletic individuals (muscle mass considerations)"""
        # Note: BMI doesn't account for muscle mass, so muscular athletes 
        # might be classified as overweight despite being healthy
        test_cases = [
            (180, 85, "overweight"),  # Muscular athlete
            (175, 80, "overweight"),  # Strong athlete
        ]
        
        for height, weight, expected_category in test_cases:
            bmi = calc_bmi(height, weight)
            result = print_bmi(bmi)
            assert expected_category in result

    def test_documentation_examples(self):
        """Test examples that could be used in documentation"""
        # WHO standard examples
        bmi_18_5 = calc_bmi(170, 53.465)  # Should be exactly 18.5
        assert abs(bmi_18_5 - 18.5) < 0.01
        
        bmi_25 = calc_bmi(170, 72.25)  # Should be exactly 25.0
        assert abs(bmi_25 - 25.0) < 0.01
        
        bmi_30 = calc_bmi(170, 86.7)  # Should be exactly 30.0
        assert abs(bmi_30 - 30.0) < 0.01


# -*- coding: utf-8 -*-
import pytest
import sys
import os

# Add the project root to Python path
sys.path.insert(0, os.path.join(os.path.dirname(__file__), ".."))

# Import from pyfibot module directly
from pyfibot import pyfibot


class TestUtilityFunctions:
    """Test utility functions from PyFiBotFactory"""

    def setup_method(self):
        """Setup test fixtures"""
        # Create a minimal config for testing
        config = {"admins": ["admin@example.com", "*@admin.domain"]}
        self.factory = pyfibot.PyFiBotFactory(config)

    def test_to_utf8_string_input(self):
        """Test to_utf8 with string input"""
        result = self.factory.to_utf8("hello world")
        assert result == b"hello world"
        assert isinstance(result, bytes)

    def test_to_utf8_unicode_string(self):
        """Test to_utf8 with unicode characters"""
        result = self.factory.to_utf8("héllo wörld åäö")
        expected = "héllo wörld åäö".encode("UTF-8")
        assert result == expected
        assert isinstance(result, bytes)

    def test_to_utf8_bytes_input(self):
        """Test to_utf8 with bytes input (should return as-is)"""
        input_bytes = b"hello world"
        result = self.factory.to_utf8(input_bytes)
        assert result == input_bytes
        assert isinstance(result, bytes)

    def test_to_utf8_none_input(self):
        """Test to_utf8 with None input"""
        result = self.factory.to_utf8(None)
        assert result is None

    def test_to_utf8_numeric_input(self):
        """Test to_utf8 with numeric input"""
        result = self.factory.to_utf8(42)
        assert result == 42
        assert not isinstance(result, bytes)

    def test_to_unicode_bytes_input_utf8(self):
        """Test to_unicode with UTF-8 bytes"""
        input_bytes = "héllo wörld".encode("utf-8")
        result = self.factory.to_unicode(input_bytes)
        assert result == "héllo wörld"
        assert isinstance(result, str)

    def test_to_unicode_bytes_input_iso88591(self):
        """Test to_unicode with ISO-8859-1 bytes that fail UTF-8 decode"""
        # Create bytes that are valid ISO-8859-1 but invalid UTF-8
        input_bytes = b"\xe5\xe4\xf6"  # åäö in ISO-8859-1
        result = self.factory.to_unicode(input_bytes)
        assert result == "åäö"
        assert isinstance(result, str)

    def test_to_unicode_invalid_bytes(self):
        """Test to_unicode with bytes that can't be decoded as UTF-8 or ISO-8859-1"""
        # This shouldn't happen in practice, but test the fallback
        input_bytes = b"\xff\xfe\x00\x00"
        result = self.factory.to_unicode(input_bytes)
        assert isinstance(result, str)
        # Should fall back to str() conversion

    def test_to_unicode_string_input(self):
        """Test to_unicode with string input (should return as-is)"""
        input_str = "hello world"
        result = self.factory.to_unicode(input_str)
        assert result == input_str
        assert isinstance(result, str)

    def test_to_unicode_none_input(self):
        """Test to_unicode with None input"""
        result = self.factory.to_unicode(None)
        assert result == "None"
        assert isinstance(result, str)

    def test_to_unicode_numeric_input(self):
        """Test to_unicode with numeric input"""
        result = self.factory.to_unicode(42)
        assert result == "42"
        assert isinstance(result, str)

    def test_to_unicode_list_input(self):
        """Test to_unicode with list input"""
        result = self.factory.to_unicode([1, 2, 3])
        assert result == "[1, 2, 3]"
        assert isinstance(result, str)

    def test_roundtrip_conversion(self):
        """Test that to_utf8 and to_unicode are inverse operations"""
        original = "héllo wörld åäö"
        bytes_result = self.factory.to_utf8(original)
        string_result = self.factory.to_unicode(bytes_result)
        assert string_result == original

    def test_empty_string_conversion(self):
        """Test conversion of empty strings"""
        assert self.factory.to_utf8("") == b""
        assert self.factory.to_unicode(b"") == ""

    def test_special_characters(self):
        """Test with various special characters"""
        test_strings = [
            "🙂😀💯",  # Emojis
            "中文字符",  # Chinese characters
            "русский текст",  # Cyrillic
            "العربية",  # Arabic
            "→←↑↓",  # Arrows
            "¡¿¨´`",  # Special punctuation
        ]
        
        for test_str in test_strings:
            # Test roundtrip
            bytes_result = self.factory.to_utf8(test_str)
            string_result = self.factory.to_unicode(bytes_result)
            assert string_result == test_str


class TestAdminCheck:
    """Test the isAdmin functionality"""

    def test_exact_match_admin(self):
        """Test exact admin match"""
        config = {"admins": ["admin@example.com"]}
        factory = pyfibot.PyFiBotFactory(config)
        
        assert factory.isAdmin("admin@example.com") is True
        assert factory.isAdmin("user@example.com") is False

    def test_wildcard_admin(self):
        """Test wildcard admin patterns"""
        config = {"admins": ["*@admin.domain", "admin@*"]}
        factory = pyfibot.PyFiBotFactory(config)
        
        assert factory.isAdmin("anyone@admin.domain") is True
        assert factory.isAdmin("admin@anywhere.com") is True
        assert factory.isAdmin("user@regular.domain") is False

    def test_multiple_admin_patterns(self):
        """Test multiple admin patterns"""
        config = {"admins": ["admin1@test.com", "*@secure.net", "root@*"]}
        factory = pyfibot.PyFiBotFactory(config)
        
        assert factory.isAdmin("admin1@test.com") is True
        assert factory.isAdmin("user@secure.net") is True
        assert factory.isAdmin("root@anywhere.org") is True
        assert factory.isAdmin("user@test.com") is False

    def test_no_admins_configured(self):
        """Test when no admins are configured"""
        config = {"admins": []}
        factory = pyfibot.PyFiBotFactory(config)
        
        assert factory.isAdmin("anyone@anywhere.com") is False

    def test_complex_wildcard_patterns(self):
        """Test complex wildcard patterns"""
        config = {"admins": ["admin*@*.example.com", "*admin@test.*"]}
        factory = pyfibot.PyFiBotFactory(config)
        
        assert factory.isAdmin("admin123@sub.example.com") is True
        assert factory.isAdmin("superadmin@test.org") is True
        assert factory.isAdmin("user@sub.example.com") is False
        assert factory.isAdmin("admin@wrongdomain.com") is False

    def test_case_sensitivity(self):
        """Test case sensitivity in admin matching"""
        config = {"admins": ["Admin@Example.Com"]}
        factory = pyfibot.PyFiBotFactory(config)
        
        # fnmatch is case-sensitive by default
        assert factory.isAdmin("Admin@Example.Com") is True
        assert factory.isAdmin("admin@example.com") is False


class TestHostnameExtraction:
    """Test hostname extraction utilities"""

    def test_getHost_basic(self):
        """Test basic hostname extraction"""
        config = {"admins": []}
        factory = pyfibot.PyFiBotFactory(config)
        
        assert factory.getHost("user@example.com") == "example.com"
        assert factory.getHost("nick@host.domain.org") == "host.domain.org"

    def test_getHost_no_at_symbol(self):
        """Test hostname extraction with no @ symbol"""
        config = {"admins": []}
        factory = pyfibot.PyFiBotFactory(config)
        
        # Should return the whole string if no @ found
        result = factory.getHost("justnick")
        # This will split on @ and take [1], which doesn't exist, so it might error
        # Let's check what actually happens
        with pytest.raises(IndexError):
            factory.getHost("justnick")

    def test_getHost_multiple_at_symbols(self):
        """Test hostname extraction with multiple @ symbols"""
        config = {"admins": []}
        factory = pyfibot.PyFiBotFactory(config)
        
        # Should split on first @ and take everything after
        assert factory.getHost("user@with@multiple@ats") == "with@multiple@ats"

    def test_getIdent_basic(self):
        """Test basic ident extraction"""
        config = {"admins": []}
        factory = pyfibot.PyFiBotFactory(config)
        
        assert factory.getIdent("nick!ident@host.com") == "ident"
        assert factory.getIdent("user!~username@example.org") == "~username"

    def test_getIdent_no_exclamation(self):
        """Test ident extraction with no ! symbol"""
        config = {"admins": []}
        factory = pyfibot.PyFiBotFactory(config)
        
        # Should return the whole string if no ! found
        with pytest.raises(IndexError):
            factory.getIdent("nickonly")

    def test_getIdent_complex_format(self):
        """Test ident extraction with complex IRC format"""
        config = {"admins": []}
        factory = pyfibot.PyFiBotFactory(config)
        
        # Test various IRC user formats
        assert factory.getIdent("nick!ident@host") == "ident"
        assert factory.getIdent("user!~ident@cloak.host.net") == "~ident"


# -*- coding: utf-8 -*-
from pyfibot.util.pyfiurl import grab

needScheme = True


def testLeadingSpaces():
    """Leading spaces before URL"""
    assert ["http://tomtom.foobar.org/"] == grab(
        "     http://tomtom.foobar.org/", needScheme
    )
    assert ["http://www.foobi.org/saatoimia"] == grab(
        "  http://www.foobi.org/saatoimia", needScheme
    )


def testTrailingSpaces():
    """Trailing spaces after URL"""
    assert ["http://tomtom.foobar.org/"] == grab(
        "http://tomtom.foobar.org/     ", needScheme
    )
    assert ["http://www.foobi.org/saatoimia"] == grab(
        "http://www.foobi.org/saatoimia    ", needScheme
    )


def testLongURL():
    """A long URL-like string"""
    assert [] == grab(
        "www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www.www",
        needScheme,
    )


def testQuestionMarkURI():
    """An URI with a question mark"""
    assert ["http://www.bdog.fi/cgi-bin/netstore/tuotehaku.pl?tuoteryhma=16"] == grab(
        "http://www.bdog.fi/cgi-bin/netstore/tuotehaku.pl?tuoteryhma=16", needScheme
    )


def testLeadingText():
    """Leading text"""
    assert ["http://www.technikfoo.de/mai"] == grab(
        "here it is: http://www.technikfoo.de/mai", needScheme
    )


def testLeadingAndTrailingText():
    """Leading and trailing text"""
    assert ["http://123.123.123.123"] == grab(
        "fooasdf asdf a http://123.123.123.123 asdfasdf", needScheme
    )


def testIP():
    """http URI with an ip number"""
    assert ["http://234.234.234.234"] == grab("http://234.234.234.234", needScheme)


def testFoobarIP():
    """ip number-like text"""
    assert [] == grab(
        "http://11123.123.123.123/eisaa http://123.123.123.12345/eisaa", needScheme
    )


def test2URIs():
    """2 URIs on same text"""
    assert ["http://foobar.fi/1234{}[]{}", "http://127.0.0.1/"] == grab(
        "http://foobar.fi/1234{}[]{} sadfljs dlfkjsd lf;asdf http://127.0.0.1/",
        needScheme,
    )


def testIPv6():
    """IPv6 URL with scheme"""
    assert ["http://[2001:a68:104:1337:250:daff:fe72:871c]/toimia"] == grab(
        "foo http://[2001:a68:104:1337:250:daff:fe72:871c]/toimia", needScheme
    )


def testIPv6noscheme():
    """IPv6 URL without a scheme"""
    if needScheme:
        return
    assert ["[2001:a68:104:1337:250:daff:fe72:871c]/toimia"] == grab(
        "foo [2001:a68:104:1337:250:daff:fe72:871c]/toimia", needScheme
    )


def testNoScheme():
    """URI without a scheme"""
    if needScheme:
        return
    assert ["123.123.123.123"] == grab("123.123.123.123", needScheme)


def testRedirect():
    """Redirect URL"""
    assert [
        "http://rediretinmyurl.com/http://dest.url.org/1/2/3/4?434",
        "http://secondurl.com",
        "ftp://1.2.3.4/adsfasdf",
    ] == grab(
        "http://rediretinmyurl.com/http://dest.url.org/1/2/3/4?434 http://secondurl.com ftp://1.2.3.4/adsfasdf",
        needScheme,
    )


def testAnchor():
    """Link with an anchor tag"""
    assert ["http://foo.com/page.html#anchor"] == grab(
        "http://foo.com/page.html#anchor", needScheme
    )


def testScandinavian():
    """Test åäö"""
    assert [
        "http://www.hs.fi/kotimaa/artikkeli/Äidin+avovaimosta+lapsen+toinen+huoltaja+KKOn+päätöksellä/1135253379084"
    ] == grab(
        "http://www.hs.fi/kotimaa/artikkeli/Äidin+avovaimosta+lapsen+toinen+huoltaja+KKOn+päätöksellä/1135253379084",
        needScheme,
    )


def testBlocks():
    """Test blocks"""
    assert [
        "http://link1.com",
        "http://link2.com",
        "http://link3.com",
        "http://link4.com",
        "http://link5.com",
    ] == grab(
        "(http://link1.com) <http://link2.com> \"http://link3.com\" 'http://link4.com' [http://link5.com]",
        needScheme,
    )


def testUserinfo():
    """Test URLs with userinfo (user:pass@host)"""
    assert ["http://user:pass@example.com"] == grab(
        "http://user:pass@example.com", needScheme
    )
    assert ["https://user@example.org/path"] == grab(
        "https://user@example.org/path", needScheme
    )


def testNeedSchemeSkip():
    """Test that URLs without scheme are skipped when needScheme=True"""
    # This should find no URLs because needScheme=True but no scheme is present
    assert [] == grab("example.com/path", needScheme)
    # This should find URLs with scheme
    assert ["http://example.com/path"] == grab("http://example.com/path", needScheme)


def testUserinfoVariants():
    """Test various userinfo patterns"""
    assert ["http://user:password@host.com"] == grab(
        "http://user:password@host.com", needScheme
    )
    assert ["ftp://user@ftp.example.com/file"] == grab(
        "ftp://user@ftp.example.com/file", needScheme
    )



from nose.tools import eq_

import bot_mock


def test_example():
    assert True


def test_botmock():
    bot = bot_mock.BotMock({})
    eq_(("#channel", "Foo"), bot.say("#channel", "Foo"))


def test_factorymock():
    factory = bot_mock.FactoryMock({})
    bot = factory.find_bot_for_network("nerv")
    eq_("nerv", bot.factory.find_bot_for_network("nerv").network.alias)
    eq_("localhost", bot.factory.find_bot_for_network("localhost").network.alias)
    eq_(None, bot.factory.find_bot_for_network("not_found"))


def test_module_bmi():
    from pyfibot.modules import module_bmi

    bot = bot_mock.BotMock()
    eq_(
        ("#channel", "your bmi is 27.76 which is overweight (from 25 to 30)"),
        module_bmi.command_bmi(bot, None, "#channel", "175/85"),
    )



# -*- coding: utf-8 -*-
"""
A dictionary difference calculator
Originally posted as:
http://stackoverflow.com/questions/1165352/fast-comparison-between-two-python-dictionary/1165552#1165552
copied from https://github.com/hughdbrown/dictdiffer
"""

from typing import Dict, Set, Any


class DictDiffer(object):
    """
    Calculate the difference between two dictionaries as:
    (1) items added
    (2) items removed
    (3) keys same in both but changed values
    (4) keys same in both and unchanged values
    """

    def __init__(self, current_dict: Dict[Any, Any], past_dict: Dict[Any, Any]) -> None:
        self.current_dict = current_dict
        self.past_dict = past_dict
        self.current_keys: Set[Any] = set(current_dict.keys())
        self.past_keys: Set[Any] = set(past_dict.keys())
        self.intersect: Set[Any] = self.current_keys.intersection(self.past_keys)

    def added(self) -> Set[Any]:
        return self.current_keys - self.intersect

    def removed(self) -> Set[Any]:
        return self.past_keys - self.intersect

    def changed(self) -> Set[Any]:
        return set(
            o for o in self.intersect if self.past_dict[o] != self.current_dict[o]
        )

    def unchanged(self) -> Set[Any]:
        return set(
            o for o in self.intersect if self.past_dict[o] == self.current_dict[o]
        )



# -*- coding: utf-8 -*-
"""
Example of admin only commands
"""

from __future__ import unicode_literals, print_function, division


def admin_allow(bot, user, channel, args):
    bot.say(channel, "Example of a command only allowed to an admin")



# -*- encoding: utf-8 -*-
from __future__ import unicode_literals, print_function, division
import pytvmaze
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta
import pytz
from datetime import datetime


def command_ep(bot, user, channel, args):
    """Usage: ep <series name>"""
    return command_maze(bot, user, channel, args)


def command_maze(bot, user, channel, args):
    tvm = pytvmaze.TVMaze("pyfibot")

    try:
        show = tvm.get_show(show_name=args, embed="episodes")
    except pytvmaze.exceptions.ShowNotFound:
        bot.say(channel, "Show '%s' not found" % args)
        return

    next_episode = None
    next_delta = None

    now = datetime.now(pytz.timezone("Europe/Helsinki"))

    # go through the episodes in reverse order
    for episode in reversed(show.episodes):
        # episode has id and name, but no airstamp yet (not announced)
        if not episode.airstamp:
            continue

        delta = relativedelta(parse(episode.airstamp), now)

        # episode is in the past, stop searching
        if delta.months <= 0 and delta.days <= 0 and delta.minutes <= 0:
            break
        else:
            # episode is (still) in the future
            next_episode = episode
            next_delta = delta

    # No new episodes found
    if not next_episode:
        # ended show, find latest episode
        if show.status == "Ended":
            next_episode = show.episodes.pop()
            latest_delta = relativedelta(now, parse(next_episode.airstamp))
        else:
            # Still running, ep airdate not yet known
            bot.say(channel, "No new episodes found for %s" % show.name)
            return

    show_id = "%s %s '%s'" % (
        show.name,
        "%dx%02d" % (next_episode.season_number, next_episode.episode_number),
        next_episode.title,
    )

    if show.status == "Ended":
        msg = "Latest episode of %s aired on %s (%s ago) on %s [Ended]" % (
            show_id,
            next_episode.airdate,
            _ago(latest_delta),
            show.network.name,
        )
    else:
        msg = "Next episode of {0} airs {1} ({2})".format(
            show_id, next_episode.airdate, _ago(next_delta)
        )
        # Not all shows have network info for some reason
        if show.network:
            msg = "{0} on {1}".format(msg, show.network.name)
        elif show.web_channel:
            msg = "{0} on {1}".format(msg, show.web_channel.name)

    bot.say(channel, msg.encode("UTF-8"))


def _ago(delta, exact=False):
    delta_msg = []
    if delta.years != 0:
        delta_msg.append("%d years" % delta.years)
    if delta.months != 0:
        delta_msg.append("%d months" % delta.months)
    if delta.days != 0:
        delta_msg.append("%d days" % delta.days)

    if delta.years == 0 and delta.months == 0 and delta.days == 0:
        exact = True

    if exact:
        if delta.hours != 0:
            delta_msg.append("%d hours" % delta.hours)
        if delta.minutes != 0:
            delta_msg.append("%d minutes" % delta.minutes)

    return " ".join(delta_msg)



import urllib.parse as urllib


def command_oraakkeli(bot, user, channel, args):
    """Asks a question from the oracle (http://www.lintukoto.net/viihde/oraakkeli/)"""
    if not args:
        return
    args = urllib.quote_plus(args)
    r = bot.get_url(
        "http://www.lintukoto.net/viihde/oraakkeli/index.php?kysymys=%s&html=0" % args
    )
    return bot.say(channel, "Lintukodon Oraakkeli vastaa: %s" % r.text)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals, print_function, division
import logging

log = logging.getLogger("forecast")

has_pywapi = False
try:
    import pywapi

    has_pywapi = True
except:
    log.error("Error loading library pywapi. Library not found.")


def fahrenheit_to_celcius(f):
    return (int(f) - 32) / 1.8


def command_forecast(bot, user, channel, args):
    """This module tells weather forecast for location"""
    if not has_pywapi:
        return

    result_dict = pywapi.get_weather_from_yahoo(args)
    if not all(result_dict.values()):
        bot.say(channel, "unknown location")
        return

    def format_day(day):
        return "%s: %s (%.0f°C/%.0f°C)" % (
            day["day_of_week"],
            day["condition"],
            fahrenheit_to_celcius(day["low"]),
            fahrenheit_to_celcius(day["high"]),
        )

    answerstr = "%s: " % (result_dict["forecast_information"]["city"])
    answerstr += ", ".join(format_day(day) for day in result_dict["forecasts"])

    bot.say(channel, answerstr.encode("utf-8"))



# -*- coding: utf-8 -*-
import pytest
import sys
import os
import math
from datetime import datetime, timezone, timedelta
from unittest.mock import Mock

# Add the project root to Python path
sys.path.insert(0, os.path.join(os.path.dirname(__file__), ".."))

# Import the functions we want to test
from pyfibot.modules.module_urltitle import (
    _levenshtein_distance,
    _check_redundant,
    __get_length_str,
    __get_age_str,
    __get_views,
    __escaped_fragment,
    _title
)


class TestLevenshteinDistance:
    """Test the Levenshtein distance calculation"""

    def test_identical_strings(self):
        """Test distance between identical strings"""
        assert _levenshtein_distance("hello", "hello") == 0
        assert _levenshtein_distance("", "") == 0
        assert _levenshtein_distance("a", "a") == 0

    def test_completely_different_strings(self):
        """Test distance between completely different strings"""
        result = _levenshtein_distance("abc", "xyz")
        assert result == 3  # All 3 characters need to be replaced

    def test_single_character_changes(self):
        """Test distance with single character modifications"""
        # Single substitution
        assert _levenshtein_distance("cat", "bat") == 1
        # Single insertion
        assert _levenshtein_distance("cat", "cats") == 1
        # Single deletion
        assert _levenshtein_distance("cats", "cat") == 1

    def test_empty_string_cases(self):
        """Test distance calculations with empty strings"""
        assert _levenshtein_distance("", "hello") == 5
        assert _levenshtein_distance("hello", "") == 5
        assert _levenshtein_distance("", "") == 0

    def test_common_transformations(self):
        """Test common string transformations"""
        # "kitten" -> "sitting" (classic example)
        assert _levenshtein_distance("kitten", "sitting") == 3
        # "saturday" -> "sunday" 
        assert _levenshtein_distance("saturday", "sunday") == 3

    def test_case_sensitivity(self):
        """Test that the function is case-sensitive"""
        assert _levenshtein_distance("Hello", "hello") == 1
        assert _levenshtein_distance("ABC", "abc") == 3

    def test_unicode_strings(self):
        """Test with unicode characters"""
        assert _levenshtein_distance("café", "cafe") == 1
        assert _levenshtein_distance("naïve", "naive") == 1

    def test_long_strings(self):
        """Test with longer strings"""
        s1 = "The quick brown fox jumps over the lazy dog"
        s2 = "The quick brown fox jumped over the lazy dog"
        assert _levenshtein_distance(s1, s2) == 1  # s -> d

    def test_complex_changes(self):
        """Test complex changes requiring multiple operations"""
        # Multiple insertions, deletions, substitutions
        result = _levenshtein_distance("intention", "execution")
        assert result == 5  # Well-known example


class TestRedundancyCheck:
    """Test URL/title redundancy checking"""

    def test_obvious_redundant_cases(self):
        """Test cases where title is obviously redundant with URL"""
        # Title contains domain name
        url = "https://example.com/article"
        title = "Example.com - Latest News"
        assert _check_redundant(url, title) is True

    def test_non_redundant_cases(self):
        """Test cases where title is not redundant"""
        url = "https://news.ycombinator.com/item?id=123"
        title = "Interesting Article About Programming"
        assert _check_redundant(url, title) is False

    def test_www_subdomain_handling(self):
        """Test that www subdomain is properly stripped"""
        url = "https://www.example.com/page"
        title = "Example Website Content"
        # Should detect redundancy even with www
        result = _check_redundant(url, title)
        assert isinstance(result, bool)  # Function should complete without error

    def test_nordic_character_normalization(self):
        """Test Nordic character normalization"""
        url = "https://example.com/page"
        title = "Exämple Örgänizätion"
        # Should normalize ä->a, ö->o when checking redundancy
        result = _check_redundant(url, title)
        assert isinstance(result, bool)

    def test_case_insensitive_comparison(self):
        """Test that comparison is case-insensitive"""
        url = "https://EXAMPLE.COM/page"
        title = "example news article"
        result = _check_redundant(url, title)
        assert isinstance(result, bool)

    def test_port_number_handling(self):
        """Test URLs with port numbers"""
        url = "https://example.com:8080/page"
        title = "Example Site Content"
        result = _check_redundant(url, title)
        assert isinstance(result, bool)

    def test_subdomain_in_title(self):
        """Test when subdomain appears in title"""
        url = "https://blog.example.com/post"
        title = "Blog Example Post"
        result = _check_redundant(url, title)
        assert isinstance(result, bool)


class TestLengthString:
    """Test time duration formatting"""

    def test_zero_seconds(self):
        """Test zero duration"""
        assert __get_length_str(0) == "0s"

    def test_seconds_only(self):
        """Test durations less than a minute"""
        assert __get_length_str(30) == "30s"
        assert __get_length_str(59) == "59s"

    def test_minutes_only(self):
        """Test durations in minutes"""
        assert __get_length_str(60) == "1m"
        assert __get_length_str(90) == "1m30s"
        assert __get_length_str(120) == "2m"

    def test_hours_only(self):
        """Test durations in hours"""
        assert __get_length_str(3600) == "1h"
        assert __get_length_str(7200) == "2h"

    def test_complex_durations(self):
        """Test complex durations with hours, minutes, and seconds"""
        assert __get_length_str(3661) == "1h1m1s"  # 1:01:01
        assert __get_length_str(3720) == "1h2m"    # 1:02:00
        assert __get_length_str(3665) == "1h1m5s"  # 1:01:05

    def test_very_long_durations(self):
        """Test very long durations"""
        day_seconds = 24 * 3600
        assert __get_length_str(day_seconds) == "24h"
        assert __get_length_str(day_seconds + 3661) == "25h1m1s"

    def test_edge_cases(self):
        """Test edge cases"""
        assert __get_length_str(1) == "1s"
        assert __get_length_str(61) == "1m1s"
        assert __get_length_str(3601) == "1h1s"


class TestAgeString:
    """Test age/time delta formatting"""

    def test_recent_time(self):
        """Test very recent times"""
        now = datetime.now(timezone.utc)
        recent = now - timedelta(seconds=30)
        result = __get_age_str(recent)
        assert "fresh" in result or "0m" in result

    def test_minutes_ago(self):
        """Test times in minutes"""
        now = datetime.now(timezone.utc)
        past = now - timedelta(minutes=5)
        result = __get_age_str(past)
        assert "5m" in result

    def test_hours_ago(self):
        """Test times in hours"""
        now = datetime.now(timezone.utc)
        past = now - timedelta(hours=2)
        result = __get_age_str(past)
        assert "2h" in result

    def test_days_ago(self):
        """Test times in days"""
        now = datetime.now(timezone.utc)
        past = now - timedelta(days=3)
        result = __get_age_str(past)
        assert "3d" in result or "72h" in result

    def test_future_time(self):
        """Test future times"""
        now = datetime.now(timezone.utc)
        future = now + timedelta(hours=1)
        result = __get_age_str(future)
        # Should handle future times gracefully
        assert isinstance(result, str)

    def test_timezone_aware(self):
        """Test with timezone-aware datetimes"""
        utc_tz = timezone.utc
        now = datetime.now(utc_tz)
        past = now - timedelta(hours=1)
        result = __get_age_str(past)
        assert "1h" in result or "60m" in result


class TestViews:
    """Test view count formatting"""

    def test_zero_views(self):
        """Test zero views"""
        assert __get_views(0) == "0"

    def test_small_numbers(self):
        """Test small view counts"""
        assert __get_views(42) == "42"
        assert __get_views(999) == "999"

    def test_thousands(self):
        """Test thousands formatting"""
        assert __get_views(1000) == "1k"
        assert __get_views(1500) == "2k"  # Rounded
        assert __get_views(999000) == "999k"

    def test_millions(self):
        """Test millions formatting"""
        assert __get_views(1000000) == "1M"
        assert __get_views(1500000) == "2M"
        assert __get_views(999000000) == "999M"

    def test_billions(self):
        """Test billions formatting"""
        assert __get_views(1000000000) == "1Billion"
        assert __get_views(1500000000) == "2Billion"

    def test_trillions(self):
        """Test trillions formatting"""
        assert __get_views(1000000000000) == "1Trillion"

    def test_float_inputs(self):
        """Test with float inputs"""
        assert __get_views(1500.7) == "2k"
        assert __get_views(1000000.5) == "1M"

    def test_string_inputs(self):
        """Test with string inputs that can be converted to int"""
        assert __get_views("1000") == "1k"
        assert __get_views("1000000") == "1M"

    def test_edge_cases(self):
        """Test edge cases"""
        assert __get_views(1) == "1"
        assert __get_views(10) == "10"
        assert __get_views(100) == "100"


class TestEscapedFragment:
    """Test URL escaped fragment handling"""

    def test_no_fragment(self):
        """Test URLs without fragments"""
        url = "https://example.com/page"
        result = __escaped_fragment(url)
        assert result == url

    def test_regular_fragment(self):
        """Test URLs with regular fragments (not starting with !)"""
        url = "https://example.com/page#section"
        result = __escaped_fragment(url)
        assert result == url

    def test_ajax_fragment(self):
        """Test URLs with AJAX fragments (starting with #!)"""
        url = "https://example.com/page#!state=123"
        result = __escaped_fragment(url)
        assert "_escaped_fragment_=state=123" in result
        assert "#!" not in result

    def test_ajax_fragment_with_query(self):
        """Test URLs with existing query parameters and AJAX fragments"""
        url = "https://example.com/page?param=value#!state=123"
        result = __escaped_fragment(url)
        assert "_escaped_fragment_=state=123" in result
        assert "param=value" in result
        assert "&" in result  # Should join with &

    def test_empty_ajax_fragment(self):
        """Test URLs with empty AJAX fragments"""
        url = "https://example.com/page#!"
        result = __escaped_fragment(url)
        assert "_escaped_fragment_=" in result

    def test_complex_ajax_fragment(self):
        """Test URLs with complex AJAX fragments"""
        url = "https://example.com/page#!route=/user/123/profile"
        result = __escaped_fragment(url)
        assert "_escaped_fragment_=route=/user/123/profile" in result

    def test_meta_parameter(self):
        """Test the meta parameter behavior"""
        url = "https://example.com/page#section"
        result = __escaped_fragment(url, meta=True)
        # When meta=True and no AJAX fragment, might return None
        assert result is None or isinstance(result, str)


class MockBot:
    """Mock bot for testing _title function"""
    def __init__(self):
        self.messages = []
    
    def say(self, channel, message):
        self.messages.append((channel, message))
        return (channel, message)


class TestTitleFunction:
    """Test the _title function"""

    def setup_method(self):
        """Setup test fixtures"""
        self.bot = MockBot()

    def test_simple_title(self):
        """Test simple title output"""
        result = _title(self.bot, "#test", "Simple Title")
        assert result[0] == "#test"
        assert "Title: Simple Title" in result[1]

    def test_custom_prefix(self):
        """Test custom prefix"""
        result = _title(self.bot, "#test", "Custom Title", prefix="Video:")
        assert "Video: Custom Title" in result[1]

    def test_title_with_info(self):
        """Test title with additional info tuple"""
        title_tuple = ("Main Title", "Additional Info")
        result = _title(self.bot, "#test", title_tuple)
        assert "Title: Main Title [Additional Info]" in result[1]

    def test_long_title_truncation(self):
        """Test that very long titles are truncated"""
        long_title = "a" * 500  # Longer than 400 chars
        result = _title(self.bot, "#test", long_title)
        assert len(result[1]) < 450  # Should be truncated
        assert "..." in result[1]

    def test_empty_title(self):
        """Test with empty title"""
        result = _title(self.bot, "#test", "")
        assert result is None

    def test_none_title(self):
        """Test with None title"""
        result = _title(self.bot, "#test", None)
        assert result is None

    def test_title_with_url_caching(self):
        """Test title with URL for caching"""
        # This test assumes cache module is available
        try:
            result = _title(self.bot, "#test", "Test Title", url="https://example.com")
            assert result[0] == "#test"
            assert "Test Title" in result[1]
        except NameError:
            # If cache module is not available, skip this test
            pytest.skip("Cache module not available")

    def test_unicode_title(self):
        """Test with unicode characters in title"""
        unicode_title = "Tëst Tîtle with Spéciàl Chars"
        result = _title(self.bot, "#test", unicode_title)
        assert unicode_title in result[1]

    def test_html_entities_in_title(self):
        """Test title with HTML entities"""
        html_title = "Title with &amp; &lt;entities&gt;"
        result = _title(self.bot, "#test", html_title)
        assert html_title in result[1]


# -*- coding: utf-8 -*-
import pytest
from unittest.mock import Mock, patch
import sys
import os

# Add the project root to Python path to ensure imports work
sys.path.insert(0, os.path.join(os.path.dirname(__file__), ".."))

# Test the core command logic directly without complex imports
# We'll mock the botcore module to test its functionality


class MockBot:
    """Simple mock bot for testing core commands"""

    def __init__(self):
        self.factory = Mock()
        self.factory.isAdmin = Mock(return_value=True)
        self.factory.allBots = {}
        self.factory.ns = {}
        self.factory.reload_config = Mock()
        self.factory._unload_removed_modules = Mock()
        self.factory._loadmodules = Mock()

        self.network = Mock()
        self.network.alias = "testnet"
        self.network.channels = []
        self.pingAve = 0.5
        self.hasQuit = False

    def say(self, channel, message, length=None):
        """Mock say method"""
        return (channel, message)

    def quit(self, message):
        """Mock quit method"""
        pass


class TestCoreCommandsLogic:
    """Test the core command logic independently"""

    def setup_method(self):
        """Setup test fixtures"""
        self.bot = MockBot()

    def test_say_returns_tuple(self):
        """Test that say method returns expected tuple"""
        result = self.bot.say("#test", "hello")
        assert result == ("#test", "hello")

    def test_command_echo(self):
        """Test the echo command"""
        result = self.bot.command_echo("testuser", "#testchan", "hello world")
        # The say method in BotMock returns a tuple
        assert result == ("#testchan", "testuser: hello world")

    def test_command_ping(self):
        """Test the ping command"""
        self.bot.get_nick = Mock(return_value="testuser")
        result = self.bot.command_ping("testuser!test@example.com", "#testchan", "")
        expected_ping = int(self.bot.pingAve * 100.0)
        assert result == (
            "#testchan",
            f"testuser: My current ping is {expected_ping}ms",
        )

    def test_command_rehash_admin_success(self):
        """Test rehash command with admin privileges - success case"""
        with patch("pyfibot.botcore.rebuild.updateInstance") as mock_update:
            with patch("pyfibot.botcore.log") as mock_log:
                result = self.bot.command_rehash(
                    "admin!test@example.com", "#testchan", ""
                )

                # Verify the rebuild was called
                mock_update.assert_called_once_with(self.bot)
                # Verify modules were reloaded
                self.bot.factory._unload_removed_modules.assert_called_once()
                self.bot.factory._loadmodules.assert_called_once()
                # Verify success message
                assert result == ("#testchan", "Rehash OK")

    def test_command_rehash_with_conf(self):
        """Test rehash command with conf argument"""
        with patch("pyfibot.botcore.rebuild.updateInstance"):
            result = self.bot.command_rehash(
                "admin!test@example.com", "#testchan", "conf"
            )
            # Config reload should be called
            self.bot.factory.reload_config.assert_called_once()
            assert result == ("#testchan", "Configuration reloaded.")

    def test_command_rehash_non_admin(self):
        """Test rehash command without admin privileges"""
        self.bot.factory.isAdmin.return_value = False
        result = self.bot.command_rehash("user!test@example.com", "#testchan", "")
        # Should return None (no action taken)
        assert result is None

    def test_command_rehash_exception(self):
        """Test rehash command when an exception occurs"""
        with patch(
            "pyfibot.botcore.rebuild.updateInstance",
            side_effect=Exception("Test error"),
        ):
            with patch("pyfibot.botcore.log") as mock_log:
                result = self.bot.command_rehash(
                    "admin!test@example.com", "#testchan", ""
                )
                assert result == ("#testchan", "Rehash error: Test error")
                mock_log.error.assert_called_once()

    def test_say_not_implemented(self):
        """Test that the base say method raises NotImplementedError"""
        from pyfibot.botcore import CoreCommands

        core = CoreCommands()
        with pytest.raises(NotImplementedError):
            core.say("#test", "message")

    def test_command_join_admin_new_channel(self):
        """Test join command for a new channel"""
        # Setup mock bot for the network
        mock_bot = Mock()
        mock_bot.network = self.network
        mock_bot.network.channels = ["#existing"]
        mock_bot.join = Mock()
        self.bot.factory.allBots = {"testnet": mock_bot}

        result = self.bot.command_join(
            "admin!test@example.com", "#testchan", "#newchan"
        )
        mock_bot.join.assert_called_once_with("#newchan")

    def test_command_join_with_password(self):
        """Test join command with password"""
        mock_bot = Mock()
        mock_bot.network = self.network
        mock_bot.network.channels = []
        mock_bot.join = Mock()
        self.bot.factory.allBots = {"testnet": mock_bot}

        result = self.bot.command_join(
            "admin!test@example.com", "#testchan", "#newchan secret"
        )
        mock_bot.join.assert_called_once_with("#newchan", key="secret")

    def test_command_join_already_in_channel(self):
        """Test join command when already in channel"""
        mock_bot = Mock()
        mock_bot.network = self.network
        mock_bot.network.channels = ["#newchan"]
        self.bot.factory.allBots = {"testnet": mock_bot}

        result = self.bot.command_join(
            "admin!test@example.com", "#testchan", "#newchan"
        )
        assert result == ("#testchan", "I am already in #newchan on testnet.")

    def test_command_join_network_not_found(self):
        """Test join command with non-existent network"""
        self.bot.factory.allBots = {}
        result = self.bot.command_join(
            "admin!test@example.com", "#testchan", "#newchan@badnet"
        )
        assert result == ("#testchan", "I am not on that network.")

    def test_command_join_non_admin(self):
        """Test join command without admin privileges"""
        self.bot.factory.isAdmin.return_value = False
        result = self.bot.command_join("user!test@example.com", "#testchan", "#newchan")
        assert result is None

    def test_command_part_success(self):
        """Test part command success"""
        mock_bot = Mock()
        mock_bot.network = self.network
        mock_bot.network.channels = ["#testchan"]
        mock_bot.part = Mock()
        self.bot.factory.allBots = {"testnet": mock_bot}

        self.bot.command_part("admin!test@example.com", "#testchan", "#testchan")
        mock_bot.part.assert_called_once_with("#testchan")
        assert "#testchan" not in mock_bot.network.channels

    def test_command_part_not_in_channel(self):
        """Test part command when not in channel"""
        mock_bot = Mock()
        mock_bot.network = self.network
        mock_bot.network.channels = ["#other"]
        self.bot.factory.allBots = {"testnet": mock_bot}

        with patch("pyfibot.botcore.log") as mock_log:
            self.bot.command_part("admin!test@example.com", "#testchan", "#nothere")
            mock_log.debug.assert_called()

    def test_command_leave_alias(self):
        """Test that leave is an alias for part"""
        with patch.object(self.bot, "command_part") as mock_part:
            self.bot.command_leave("user", "#chan", "args")
            mock_part.assert_called_once_with("user", "#chan", "args")

    def test_command_quit_admin(self):
        """Test quit command with admin privileges"""
        self.bot.quit = Mock()
        self.bot.command_quit("admin!test@example.com", "#testchan", "")
        self.bot.quit.assert_called_once_with("Working as programmed")
        assert self.bot.hasQuit == 1

    def test_command_quit_non_admin(self):
        """Test quit command without admin privileges"""
        self.bot.factory.isAdmin.return_value = False
        self.bot.quit = Mock()
        result = self.bot.command_quit("user!test@example.com", "#testchan", "")
        self.bot.quit.assert_not_called()
        assert result is None

    def test_command_channels_no_args(self):
        """Test channels command without arguments"""
        self.bot.factory.allBots = {"net1": Mock(), "net2": Mock()}
        result = self.bot.command_channels("user", "#testchan", "")
        assert result == ("#testchan", "Please specify a network: net1, net2")

    def test_command_channels_with_args(self):
        """Test channels command with network argument"""
        self.bot.network.channels = ["#chan1", "#chan2"]
        result = self.bot.command_channels("user", "#testchan", "testnet")
        assert result == ("#testchan", "I am on ['#chan1', '#chan2']")

    def test_command_help_no_args(self):
        """Test help command without arguments"""
        # Mock namespace with some commands
        self.bot.factory.ns = {
            "module1": (
                {},
                {
                    "command_test": Mock(__doc__="Test command"),
                    "admin_admin": Mock(__doc__="Admin command"),
                },
            )
        }
        self.bot.factory.isAdmin.return_value = True

        result = self.bot.command_help("admin!test@example.com", "#testchan", "")
        assert result == ("#testchan", "Available commands: test")

    def test_command_help_specific_command(self):
        """Test help command for specific command"""
        mock_cmd = Mock()
        mock_cmd.__doc__ = "Test command does something\nMore details here"

        self.bot.factory.ns = {"module1": ({}, {"command_test": mock_cmd})}

        result = self.bot.command_help("user", "#testchan", "test")
        assert result == ("#testchan", "Help for test: Test command does something")


class TestBotMock:
    """Test the BotMock class functionality"""

    def test_bot_mock_initialization(self):
        """Test BotMock initialization"""
        config = {"cmdchar": "!"}

        # Test basic initialization
        bot = bot_mock.BotMock(config)
        assert bot.config == config

    def test_bot_mock_say(self):
        """Test BotMock say method"""
        bot = bot_mock.BotMock()
        result = bot.say("#channel", "test message")
        assert result == ("#channel", "test message")



# -*- coding: utf-8 -*-
"""
Koronavirus statistics from HS.fi open data
https://github.com/HS-Datadesk/koronavirus-avoindata
"""

from __future__ import unicode_literals, print_function, division
from collections import Counter
from datetime import date
import dateutil.parser


def init(bot):
    global lang
    config = bot.config.get("module_posti", {})
    lang = config.get("language", "en")


def command_korona(bot, user, channel, args):
    """Get latest info about COVID-19 in Finland (Source: https://github.com/HS-Datadesk/koronavirus-avoindata )"""

    # Generic data
    url = "https://w3qa5ydb4l.execute-api.eu-west-1.amazonaws.com/prod/finnishCoronaData/v2"
    try:
        r = bot.get_url(url)
        data = r.json()
    except Exception as e:
        bot.say(
            channel,
            "Error while getting data from API",
        )
        raise e

    if data.get("message", "") == "Internal server error":
        bot.say(
            channel,
            "API Error: %s (Blame HS Datadesk at: https://github.com/HS-Datadesk/koronavirus-avoindata )"
            % data.get("message", "Unknown"),
        )
        return

    # Data from THL, via HS datadesk since the THL api is a pain
    url = "https://w3qa5ydb4l.execute-api.eu-west-1.amazonaws.com/prod/finnishCoronaHospitalData"
    try:
        r = bot.get_url(url)
        data2 = r.json()
    except Exception as e:
        bot.say(
            channel,
            "Error while getting data from API",
        )
        raise e

    # Filter full country stats
    hospitalised = filter(lambda x: x["area"] == "Finland", data2["hospitalised"])

    # calculate total + today's increase
    total_hospitalised = hospitalised[-1]["totalHospitalised"]
    total_ward = hospitalised[-1]["inWard"]
    total_icu = hospitalised[-1]["inIcu"]
    today_hospitalised = total_hospitalised - hospitalised[-2]["totalHospitalised"]
    today_ward = total_ward - hospitalised[-2]["inWard"]
    today_icu = total_icu - hospitalised[-2]["inIcu"]

    today_confirmed = len(
        list(
            filter(
                lambda x: dateutil.parser.parse(x["date"]).date() == date.today(),
                data["confirmed"],
            )
        )
    )
    today_deaths = len(
        list(
            filter(
                lambda x: dateutil.parser.parse(x["date"]).date() == date.today(),
                data["deaths"],
            )
        )
    )
    today_recovered = len(
        list(
            filter(
                lambda x: dateutil.parser.parse(x["date"]).date() == date.today(),
                data["recovered"],
            )
        )
    )

    # dict for printing fancily
    display = {
        "confirmed": len(data["confirmed"]),
        "deaths": len(data["deaths"]),
        "today_confirmed": today_confirmed,
        "today_deaths": today_deaths,
        "today_recovered": today_recovered,
        "total_hospitalised": total_hospitalised,
        "total_ward": total_ward,
        "total_icu": total_icu,
        "today_hospitalised": today_hospitalised,
        "today_ward": today_ward,
        "today_icu": today_icu,
    }

    msg = "[COVID-19 SUOMESSA]"
    msg += " Vahvistettuja tapauksia: {confirmed}, Kuolleita: {deaths} ({today_deaths:+d}), Osastolla: {total_hospitalised} ({today_hospitalised:+d}), joista teholla: {total_icu} ({today_icu:+d})".format(
        **display
    )

    # top5 infection sources
    top5 = Counter(
        map(lambda x: x["infectionSourceCountry"], data["confirmed"])
    ).most_common(5)

    topstr = []
    for country, count in top5:
        if country is None:
            country = "N/A"

        topstr.append(country + ":" + str(count))

    # msg = msg + | Top5 lähdemaat: " + " ".join(topstr)

    bot.say(channel, msg)



from __future__ import unicode_literals, print_function, division
import requests
from typing import Dict, Any

client_id: str = "a7a5d6bc929d48f"
client_secret: str = "57b1f90a12d4d72762b4b1bf644af5157f73fed5"

print("Open the following URL in a browser:")
print(
    "https://api.imgur.com/oauth2/authorize?client_id=%s&response_type=pin&state=auth1"
    % client_id
)

pin: str = input("enter pin:")
r: requests.Response = requests.post(
    "https://api.imgur.com/oauth2/token",
    data={
        "client_id": client_id,
        "client_secret": client_secret,
        "grant_type": "pin",
        "pin": pin,
    },
)

response_data: Dict[str, Any] = r.json()

print("Paste the following to your configuration:")

print(
    """
module_imgur:
  album_id: YOUR_ALBUM_ID_HERE
  access_token: %s
  refresh_token: %s
"""
    % (response_data["access_token"], response_data["refresh_token"])
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division
import urllib.parse as urllib
import logging
import re
import xml.etree.ElementTree as etree

log = logging.getLogger("wolfram_alpha")


appid = None
query = "http://api.wolframalpha.com/v2/query?input=%s&appid=%s"


def init(bot):
    global appid
    config = bot.config.get("module_wolfram_alpha", {})
    appid = config.get("appid", "")
    if appid:
        log.info("Using Wolfram Alpha appid %s" % appid)
    else:
        log.warning("Appid not found from config!")


def clean_question(_string):
    return clean_answer(_string)


def clean_answer(_string):
    if _string:
        res = re.sub(
            "[ ]{2,}",
            " ",
            _string.replace(" | ", ": ").replace("\n", " | ").replace("~~", " ≈ "),
        ).strip()
        res = res.replace(r"\:0e3f", "฿")
        res = res.replace(r"\:ffe5", "￥")
        return res


def command_wa(bot, user, channel, args):
    """Query Wolfram Alpha"""
    if not appid:
        log.warn("Appid not specified in configuration!")
        return False

    r = bot.get_url(query % (urllib.quote(args.encode("utf-8")), appid))

    if r.status_code != 200:
        return

    root = etree.fromstring(r.content)
    # Find all pods with plaintext answers
    # Filter out None -answers, strip strings and filter out the empty ones
    plain_text_pods = filter(
        None,
        [
            p.text.strip()
            for p in root.findall(".//subpod/plaintext")
            if p is not None and p.text is not None
        ],
    )

    # no answer pods found, check if there are didyoumeans-elements
    if not plain_text_pods:
        didyoumeans = root.find("didyoumeans")
        # no support for future stuff yet, TODO?
        if not didyoumeans:
            # If there's no pods, the question clearly wasn't understood
            return bot.say(channel, "Sorry, couldn't understand the question.")

        options = []
        for didyoumean in didyoumeans:
            options.append("'%s'" % didyoumean.text)
        line = " or ".join(options)
        line = "Did you mean %s?" % line
        return bot.say(channel, line.encode("UTF-8"))

    # If there's only one pod with text, it's probably the answer
    # example: "integral x²"
    if len(plain_text_pods) == 1:
        answer = clean_question(plain_text_pods[0])
        return bot.say(channel, answer)

    # If there's multiple pods, first is the question interpretation
    question = clean_question(plain_text_pods[0])
    # and second is the best answer
    answer = clean_answer(plain_text_pods[1])
    return bot.say(channel, "%s = %s" % (question, answer))



# -*- encoding: utf-8 -*-
"""
Get consignment tracking info from Matkahuolto
"""

from __future__ import unicode_literals, print_function, division
from datetime import datetime
from bs4 import BeautifulSoup


def init(bot):
    global lang
    config = bot.config.get("module_matkahuolto", {})
    lang = config.get("language", "en")


def command_mh(bot, user, channel, args):
    """Get latest consignment status from Matkahuolto Track & Trace service"""
    params = {"package_code": args}

    url = "https://www.matkahuolto.fi/%s/seuranta/tilanne/" % lang

    try:
        r = bot.get_url(url, params=params)
        r.raise_for_status()
        bs = BeautifulSoup(r.content)

        events = bs.select(".events-table table tbody tr")
        if not events:
            alert = bs.select(".tracker-status .alert")[0].get_text(strip=True)
            return bot.say(channel, alert)

    except Exception as e:
        bot.say(
            "Error while getting tracking data. Check the tracking ID or try again later."
        )
        raise e

    latest_event = events[0].select("td")

    datestr = latest_event[0].get_text(strip=True)
    status = latest_event[1].get_text(strip=True)
    place = latest_event[2].stripped_strings.next()

    dt = datetime.now() - datetime.strptime(datestr, "%d.%m.%Y, %H:%M")

    agestr = []
    if dt.days > 0:
        agestr.append("%dd" % dt.days)
    secs = dt.seconds
    hours, minutes = secs // 3600, secs // 60 % 60
    if hours > 0:
        agestr.append("%dh" % hours)
    if minutes > 0:
        agestr.append("%dm" % minutes)

    ago = "%s %s" % (
        " ".join(agestr),
        {"fi": "sitten", "sv": "sedan", "en": "ago"}[lang],
    )

    return bot.say(channel, " - ".join([ago, status, place]))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division
import dataset
import logging
import re
from datetime import datetime
from dateutil.parser import parse as parse_dt
from copy import deepcopy
import os


log = logging.getLogger("usertrack")

if not os.path.exists("databases"):
    os.makedirs("databases")
db = dataset.connect("sqlite:///databases/usertrack.db")


def get_table(bot, channel):
    """Returns table-instance from database.
    Database names are in format "networkalias_channel".
    Network alias is the name assigned to network in config.
    Channel is stripped from the &#!+ prepending the channel name."""

    return db["%s_%s" % (bot.network.alias, re.sub(r"&|#|!|\+", "", channel))]


def upsert_row(bot, channel, data, keys=["nick", "ident", "host"]):
    """Updates row to database.
    Default keys are nick, ident and host,
    which are normally present for data received by get_base_data -function."""

    table = get_table(bot, channel)
    table.upsert(data, keys)


def get_base_data(user):
    """Fetches "base" data according to user.
    Normally this is nick, ident, host and current time to be set to action_time in database.
    Ident and host might be missing when the full mask isn't provided,
    for example in handle_userKicked, where kickee doesn't get anything but name."""

    data = {"nick": getNick(user), "action_time": datetime.now()}

    # For example kickee doesn't get full hostmask -> needs some work...
    try:
        data["ident"] = getIdent(user)
    except IndexError:
        pass
    try:
        data["host"] = getHost(user)
    except IndexError:
        pass

    return data


def handle_privmsg(bot, user, channel, message):
    """Handles all messages bot sees.
    If message is private message to bot, doesn't update.
    Otherwise updates the DB.
    - last_action = 'message'
    - last_message = message
    - message_time = current time"""

    # if user == channel -> this is a query -> don't update
    if user == channel:
        return

    data = get_base_data(user)
    data["last_action"] = "message"
    data["last_message"] = message
    data["message_time"] = datetime.now()

    upsert_row(bot, channel, data)


def handle_userJoined(bot, user, channel):
    """Handles user joining the channel and auto-ops if op == True in database.
    - last_action = 'join'"""

    data = get_base_data(user)
    data["last_action"] = "join"

    upsert_row(bot, channel, data)

    table = get_table(bot, channel)
    if table.find_one(
        nick=getNick(user), ident=getIdent(user), host=getHost(user), op=True
    ):
        log.info("auto-opping %s" % user)
        bot.mode(channel, True, "o", user=getNick(user))


def handle_userLeft(bot, user, channel, message):
    """Handles user leaving the channel (or quitting).
    For leaving, only updates the channel left.
    For quitting, updates all channels in network, which the user was on (as bot knows...)
    - last_action = 'left'"""

    data = get_base_data(user)
    data["last_message"] = message
    if channel is not None:
        data["last_action"] = "left"
        upsert_row(bot, channel, data)
    # QUIT returns the channel as None, loop through all tables, check if user exists and update it
    else:
        data["last_action"] = "quit"
        for t in db.tables:
            if not t.startswith("%s_" % bot.network.alias):
                continue
            table = db.load_table(t)
            res = table.find_one(
                nick=getNick(user), ident=getIdent(user), host=getHost(user)
            )
            if res:
                data["id"] = res["id"]
                table.update(data, ["id"])


def handle_userKicked(bot, kickee, channel, kicker, message):
    """Handles user being kicked.
    As 'kickee' doesn't get full mask, it's only determined by nick.
    For kickee:
        - last_action = kicked by kicker [message]
    For kicker:
        - last_action = kicked kickee [message]"""

    data = get_base_data(kickee)
    data["last_action"] = "kicked by %s [%s]" % (getNick(kicker), message)
    # We don't get full info from kickee, need to update by nick only
    upsert_row(bot, channel, data, ["nick"])

    # Update the kickers action also...
    data = get_base_data(kicker)
    data["last_action"] = "kicked %s [%s]" % (getNick(kickee), message)
    upsert_row(bot, channel, data)


def handle_userRenamed(bot, user, newnick):
    """Handles nick change.
    Updates both data, related to old and new nick, doesn't remove anything from db.
    - last_action = nick change from oldnick to newnick"""

    nick = getNick(user)
    ident = getIdent(user)
    host = getHost(user)

    data = get_base_data(user)
    data["last_action"] = "nick change from %s to %s" % (nick, newnick)

    # loop through all the tables, if user exists, update nick to match
    for t in db.tables:
        if not t.startswith("%s_" % bot.network.alias):
            continue

        table = db.load_table(t)

        # if row is found with new or old nick -> user is on the channel -> update
        if table.find_one(nick=nick, ident=ident, host=host) or table.find_one(
            nick=newnick, ident=ident, host=host
        ):
            # need to create a deep copy of data, as dataset seems to put changed fields back to data...
            # haven't found any documentation on this, so might be a bug?
            tmp_data = deepcopy(data)

            # update the old user
            table.upsert(tmp_data, ["nick", "ident", "host"])

            # update new user
            tmp_data = deepcopy(data)
            tmp_data["nick"] = newnick
            table.upsert(tmp_data, ["nick", "ident", "host"])


def handle_action(bot, user, channel, message):
    """Handles action (/me etc). Ignores stuff directed to bot (/describe botnick etc).
    - last_action = action
    - last_message = message
    - message_time = current time"""

    # if action is directed to bot instead of channel -> don't log
    if channel == bot.nickname:
        return

    data = get_base_data(user)
    data["last_action"] = "action"
    data["last_message"] = message
    data["message_time"] = datetime.now()

    upsert_row(bot, channel, data)


def command_add_op(bot, user, channel, args):
    """Adds op-status according to nickname or full hostmask. Only for admins.
    If user is found from database, set op = True and return info with full hostmask.
    Else returns user not found."""

    if not isAdmin(user) or user == channel or not args:
        return

    table = get_table(bot, channel)

    # Get basedata for user to be opped
    u = get_base_data(args)

    # If we got full mask, use it..
    if "nick" in u and "ident" in u and "host" in u:
        res = table.find_one(nick=u["nick"], ident=u["ident"], host=u["host"])
    # else use just nickname
    else:
        res = table.find_one(nick=u["nick"])

    if not res:
        return bot.say(channel, "user not found")

    data = {"id": res["id"], "op": True}
    table.upsert(data, ["id"])
    return bot.say(
        channel, "auto-opping %s!%s@%s" % (res["nick"], res["ident"], res["host"])
    )


def command_remove_op(bot, user, channel, args):
    """Removes op-status from nick. Logic same as command_add_op. Only for admins."""

    if not isAdmin(user) or user == channel or not args:
        return

    table = get_table(bot, channel)

    # Get basedata for user to be opped
    u = get_base_data(args)

    # If we got full mask, use it..
    if "nick" in u and "ident" in u and "host" in u:
        res = table.find_one(nick=u["nick"], ident=u["ident"], host=u["host"])
    # else use just nickname
    else:
        res = table.find_one(nick=u["nick"])

    if not res:
        return bot.say(channel, "user not found")

    data = {"id": res["id"], "op": False}
    table.upsert(data, ["id"])
    return bot.say(
        channel,
        "removed auto-op from %s!%s@%s" % (res["nick"], res["ident"], res["host"]),
    )


def command_op(bot, user, channel, args):
    """Ops user if op = True for user or isAdmin."""

    table = get_table(bot, channel)
    if table.find_one(
        nick=getNick(user), ident=getIdent(user), host=getHost(user), op=True
    ) or isAdmin(user):
        log.info("opping %s on %s by request" % (user, channel))
        bot.mode(channel, True, "o", user=getNick(user))


def command_list_ops(bot, user, channel, args):
    """Lists ops in current channel. Only for admins.
    By default lists nicks, if args == 'full', lists full hostmask."""

    if not isAdmin(user) or user == channel:
        return

    table = get_table(bot, channel)
    if args.strip() == "full":
        ops = ", ".join(
            [
                "%s!%s@%s" % (r["nick"], r["ident"], r["host"])
                for r in table.find(op=True)
            ]
        )
    else:
        ops = ", ".join(["%s" % r["nick"] for r in table.find(op=True)])
    return bot.say(channel, "ops: %s" % ops)


def __get_length_str(secs):
    days, hours, minutes, seconds = (
        secs // 86400,
        secs // 3600,
        secs // 60 % 60,
        secs % 60,
    )

    if days > 0:
        return "%dd" % days
    if hours > 0:
        return "%dh" % hours
    if minutes > 0:
        return "%dm" % minutes
    if seconds > 0:
        return "%ds" % seconds
    return "0s"


def command_seen(bot, user, channel, args):
    """Displays the last action by the given user"""
    if not args:
        return bot.say(channel, "Please provide a nick to search...")

    table = get_table(bot, channel)

    # Return the first match, there shouldn't be multiples anyway
    user = table.find_one(nick=args)
    if not user:
        return bot.say(channel, "I haven't seen %s on %s" % (args, channel))

    if not isinstance(user["action_time"], datetime):
        user["action_time"] = parse_dt(user["action_time"])

    # Calculate last seen in seconds
    last_seen = datetime.now() - user["action_time"]
    # Get string for last seen
    last_seen = __get_length_str(last_seen.days * 86400 + last_seen.seconds)

    # If the last action was part or quit, show also the message
    if user["last_action"] in ["left", "quit"]:
        return bot.say(
            channel,
            "%s was last seen at %s (%s ago) [%s, %s]"
            % (
                user["nick"],
                "{0:%Y-%m-%d %H:%M:%S}".format(user["action_time"]),
                last_seen,
                user["last_action"],
                user["last_message"],
            ),
        )

    # Otherwise just show the time and action
    return bot.say(
        channel,
        "%s was last seen at %s (%s ago) [%s]"
        % (
            user["nick"],
            "{0:%Y-%m-%d %H:%M:%S}".format(user["action_time"]),
            last_seen,
            user["last_action"],
        ),
    )



from __future__ import unicode_literals, print_function, division
import requests
import base64
import sys
from typing import Dict, Any

if len(sys.argv) < 3:
    print("Usage: twitter_application_auth.py <consumer key> <consumer secret>")
    sys.exit(1)

consumer_key: str = sys.argv[1]
consumer_secret: str = sys.argv[2]
token: str = consumer_key + ":" + consumer_secret
encoded_token: bytes = base64.b64encode(token.encode("utf-8"))

payload: Dict[str, str] = {"grant_type": "client_credentials"}
headers: Dict[str, str] = {"Authorization": "Basic " + encoded_token.decode("utf-8")}
auth_url: str = "https://api.twitter.com/oauth2/token"
r: requests.Response = requests.post(auth_url, data=payload, headers=headers)
try:
    response_data: Dict[str, Any] = r.json()
    bearer_token: str = response_data["access_token"]
except (TypeError, KeyError):
    # Fallback for malformed response
    bearer_token = "ERROR_GETTING_TOKEN"

print("Paste the following to your config below module_urltitle")
print("twitter_bearer: '%s'" % bearer_token)



import os.path
import yaml
import logging

import requests
from pyfibot import pyfibot
from pyfibot import botcore

log = logging.getLogger("bot_mock")


class BotMock(botcore.CoreCommands):
    config = {}

    def __init__(self, config={}, network=None):
        self.config = config
        if network:
            self.network = network
            self.nickname = self.network.nickname
            self.lineRate = self.network.linerate
            self.password = self.network.password

    def get_url(self, url, nocache=False, params=None, headers=None, cookies=None):
        print("Getting url %s" % url)

        browser = (
            "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:43.0) Gecko/20100101 Firefox/43.0"
        )
        # Common session for all requests
        s = requests.session()
        s.stream = True  # Don't fetch content unless asked
        s.headers.update({"User-Agent": browser})
        # Custom headers from requester
        if headers:
            s.headers.update(headers)
        # Custom cookies from requester
        if cookies:
            s.cookies.update(cookies)

        try:
            r = s.get(url, params=params)
        except requests.exceptions.InvalidSchema:
            print("[ERROR] Invalid schema in URI: %s" % url)
            return None
        except requests.exceptions.ConnectionError:
            print("[ERROR] Connection error when connecting to %s" % url)
            return None

        size = int(r.headers.get("Content-Length", 0)) // 1024
        # log.debug("Content-Length: %dkB" % size)
        if size > 2048:
            print("[WARNING] Content too large, will not fetch: %skB %s" % (size, url))
            return None

        return r

    def say(self, channel, message, length=None):
        return (channel, message)

    def to_utf8(self, _string):
        """Convert string to UTF-8 bytes if it is a string"""
        if isinstance(_string, str):
            _string = _string.encode("UTF-8")
        return _string

    def to_unicode(self, _string):
        """Convert bytes to unicode string"""
        if isinstance(_string, bytes):
            try:
                _string = _string.decode("utf-8")
            except UnicodeDecodeError:
                try:
                    _string = _string.decode("iso-8859-1")
                except UnicodeDecodeError:
                    _string = str(_string)
        elif not isinstance(_string, str):
            _string = str(_string)
        return _string


class FactoryMock(pyfibot.PyFiBotFactory):
    protocol = BotMock

    def __init__(self, config={}):
        # run with main bot config if one exists
        main_config = os.path.join(os.path.dirname(__file__), "..", "config.yml")
        test_config = os.path.join(os.path.dirname(__file__), "test_config.yml")

        if not config or config == {}:
            if os.path.exists(main_config):
                log.debug("USING MAIN CONFIG")
                with open(main_config, "r") as f:
                    config = yaml.load(f, Loader=yaml.FullLoader)
            else:
                log.debug("USING TEST CONFIG")
                with open(test_config, "r") as f:
                    config = yaml.load(f, Loader=yaml.FullLoader)

        pyfibot.PyFiBotFactory.__init__(self, config)
        self.createNetwork(
            ("localhost", 6667), "nerv", "pyfibot", ["#pyfibot"], 0.5, None, False
        )
        self.createNetwork(
            ("localhost", 6667), "localhost", "pyfibot", ["#pyfibot"], 0.5, None, False
        )
        self.startFactory()
        self.buildProtocol(None)

    def startFactory(self):
        self.moduledir = "./pyfibot/modules/"
        self.allBots = {}

    def buildProtocol(self, address):
        # Go through all defined networks
        for network, server in self.data["networks"].items():
            p = self.protocol(network=server)
            self.allBots[server.alias] = p
            p.factory = self
